# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['annofabcli',
 'annofabcli.annotation',
 'annofabcli.annotation_specs',
 'annofabcli.common',
 'annofabcli.experimental',
 'annofabcli.filesystem',
 'annofabcli.input_data',
 'annofabcli.inspection_comment',
 'annofabcli.instruction',
 'annofabcli.job',
 'annofabcli.labor',
 'annofabcli.organization_member',
 'annofabcli.project',
 'annofabcli.project_member',
 'annofabcli.statistics',
 'annofabcli.supplementary',
 'annofabcli.task',
 'annofabcli.task_history']

package_data = \
{'': ['*'], 'annofabcli': ['data/*']}

install_requires = \
['annofabapi>=0.40,<0.41',
 'bokeh>=2.2,<3.0',
 'dictdiffer',
 'holoviews>=1.14,<2.0',
 'isodate',
 'jmespath',
 'more-itertools',
 'pandas>=1.1,<2.0',
 'pillow',
 'pyquery',
 'pyyaml',
 'requests']

entry_points = \
{'console_scripts': ['annofabcli = annofabcli.__main__:main']}

setup_kwargs = {
    'name': 'annofabcli',
    'version': '1.38.2',
    'description': 'Utility Command Line Interface for AnnoFab',
    'long_description': '[![Build Status](https://travis-ci.com/kurusugawa-computer/annofab-cli.svg?branch=master)](https://travis-ci.com/kurusugawa-computer/annofab-api-python-client)\n[![PyPI version](https://badge.fury.io/py/annofabcli.svg)](https://badge.fury.io/py/annofabcli)\n[![Python Versions](https://img.shields.io/pypi/pyversions/annofabcli.svg)](https://pypi.org/project/annofabcli/)\n\n\n\n# 概要\nAnnoFabのCLI(Command Line Interface)ツールです。\n「タスクの一括差し戻し」や、「プロジェクト間の差分表示」など、AnnoFabの画面で実施するには時間がかかる操作を、コマンドとして提供しています。\n\n# 注意\n* 作者または著作権者は、ソフトウェアに関してなんら責任を負いません。\n* 予告なく互換性のない変更がある可能性をご了承ください。\n* AnnoFabプロジェクトに大きな変更を及ぼすコマンドも存在します。間違えて実行してしまわないよう、注意してご利用ください。\n\n\n## 廃止予定\n* 2020-10-01以降：Pythonのサポートバージョンを3.6以上から、3.7以上に変更します。\n    \n\n# Requirements\n* Python 3.6+\n\n# Install\n\n```\n$ pip install annofabcli\n```\n\nhttps://pypi.org/project/annofabcli/\n\n\n## AnnoFabの認証情報の設定\nAnnoFabの認証情報を設定する方法は2つあります。\n* `.netrc`ファイル\n* 環境変数`ANNOFAB_USER_ID` , `ANNOFAB_PASSWORD`\n\n`.netrc`ファイルへの記載方法は、[annofab-api-python-client/README.md](https://github.com/kurusugawa-computer/annofab-api-python-client#netrc%E3%81%AB%E8%A8%98%E8%BC%89%E3%81%95%E3%82%8C%E3%81%9Fuser_id-password%E3%81%8B%E3%82%89%E7%94%9F%E6%88%90)を参照してください。\n\nAnnoFabの認証情報が設定されていない状態で`annofabcli`コマンドを実行すると、標準入力からAnnoFabの認証情報を入力できるようになります。\n\n```\n$ annofabcli project diff aaa bbb\nEnter AnnoFab User ID: XXXXXX\nEnter AnnoFab Password: \n```\n\nAnnoFabの認証情報の優先順位は以下の通りです。\n1. `.netrc`ファイル\n2. 環境変数\n\n## AnnoFab WebAPIのエンドポイントの設定（開発者用）\nAnnoFab WebAPIのエンドポイントを指定できます。デフォルトは https://annofab.com です。\n* コマンドライン引数`--endpoint_url`\n* 環境変数 `ANNOFAB_ENDPOINT_URL`\n\n設定したエンドポイントURLの優先順位は以下の通りです。\n1. コマンドライン引数\n2. 環境変数\n\n\n## Dockerを利用する場合\n\n```\n$ git clone https://github.com/kurusugawa-computer/annofab-cli.git\n$ cd annofab-cli\n$ chmod u+x docker-build.sh\n$ ./docker-build.sh\n\n$ docker run -it annofab-cli --help\n\n# AnnoFabの認証情報を標準入力から指定する\n$ docker run -it annofab-cli project diff prj1 prj2\nEnter AnnoFab User ID: XXXXXX\nEnter AnnoFab Password: \n\n# AnnoFabの認証情報を環境変数で指定する\n$ docker run -it -e ANNOFAB_USER_ID=XXXX -e ANNOFAB_PASSWORD=YYYYY annofab-cli project diff prj1 prj2\n```\n\n# 機能一覧\n\n|コマンド| サブコマンド                  | 内容                                                                                                     |必要なロール|\n|----|-------------------------------|----------------------------------------------------------------------------------------------------------|------------|\n|annotation| change_attributes |アノテーションの属性を変更します。                              |オーナ|\n|annotation| delete | アノテーションを削除します。                              |オーナ|\n|annotation| dump | アノテーション情報をファイルに保存します。                             |-|\n|annotation| list_count | task_idまたはinput_data_idで集約したアノテーションの個数を出力します                              |-|\n|annotation| import | アノテーションをインポートします。                             |オーナ|\n|annotation| restore |\'annotation dump\'コマンドで保存したファイルから、アノテーション情報をリストアします。                            |オーナ|\n|annotation_specs| history | アノテーション仕様の履歴一覧を出力します。                              |-|\n|annotation_specs| list_label | アノテーション仕様のラベル情報を出力します。                              |-|\n|annotation_specs| list_label_color             | アノテーション仕様から、label_nameとRGBを対応付けたJSONを出力します。                                      |-|\n|filesystem| write_annotation_image        | アノテーションzip、またはそれを展開したディレクトリから、アノテーションの画像（Semantic Segmentation用）を生成します。 |-|\n|input_data|delete             | 入力データを削除します。                                                            |オーナ|\n|input_data|list             | 入力データ一覧を出力します。                                                            |-|\n|input_data| list_merged_task | タスク一覧と結合した入力データ一覧のCSVを出力します。                                                            |オーナ/アノテーションユーザ|\n|input_data|list_with_json             | 入力データ全件ファイルから一覧を出力します。                                                            |-|\n|input_data|put             | 入力データを登録します。                                                            |オーナ|\n|input_data|update_metadata             | 入力データのメタデータを更新します。                                                            |オーナ|\n|inspection_comment| list | 検査コメントを出力します。                               |-|\n|inspection_comment| list_with_json | 検査コメント全件ファイルから一覧を出力します。                               |-|\n|inspection_comment| list_unprocessed | 未処置の検査コメントを出力します。                               |-|\n|instruction| copy             | 作業ガイドをコピーします。                                                         |チェッカー/オーナ|\n|instruction| upload             | HTMLファイルを作業ガイドとして登録します。                                                           |チェッカー/オーナ|\n|job|delete             | ジョブを削除します。                                                            |オーナ|\n|job|list             | ジョブ一覧を出力します。                                                            |-|\n|job|list_last             | 複数のプロジェクトに対して、最新のジョブを出力します。                                                            |-|\n|job|wait             | ジョブの終了を待ちます。                                                          |オーナ|\n|labor|list_worktime_by_user | ユーザごとに作業予定時間、作業実績時間を出力します。                                                          ||\n|organization_member|list             | 組織メンバ一覧を出力します。                                                            |-|\n|project| change_status                 | プロジェクトのステータスを変更します。                                                                          |オーナ|\n|project| copy                 | プロジェクトをコピーします。                                                                           |オーナ and 組織管理者/組織オーナ|\n|project| diff                 | プロジェクト間の差分を表示します。                                                                           |チェッカー/オーナ|\n|project| download                 | タスクや検査コメント、アノテーションなどをダウンロードします。                                                                           |オーナ|\n|project| list                 | プロジェクト一覧を出力します。                                                                          |-|\n|project| update_annotation_zip                 | アノテーションzipを更新します。                                                                         |オーナ/アノテーションユーザ|\n|project_member| change                  | プロジェクトメンバを変更します。|オーナ|\n|project_member| copy                  | プロジェクトメンバをコピーします。|オーナ(コピー先プロジェクトに対して)|\n|project_member| delete                  | 複数のプロジェクトからユーザを脱退させます。                                                                 |オーナ|\n|project_member| invite                  | 複数のプロジェクトに、ユーザを招待します。                                                                 |オーナ|\n|project_member| list                  | プロジェクトメンバ一覧を出力します。                                                                |-|\n|project_member| put                  | CSVに記載されたユーザを、プロジェクトメンバとして登録します。|オーナ|\n|statistics| list_annotation_count             | 各ラベル、各属性値のアノテーション数を、タスクごと/入力データごとに出力します。                                                   |-|\n|statistics| list_by_date_user             | タスク数や作業時間などの情報を、日ごとユーザごとに出力します。                                                   |オーナ/アノテーションユーザ|\n|statistics| list_cumulative_labor_time             |       タスク進捗状況を出力します。                                                    |-|\n|statistics| list_task_progress             | タスクフェーズ別の累積作業時間を出力します。                                                            |-|\n|statistics|summarize_task_count|タスクのフェーズ、ステータス、ステップごとにタスク数を出力します。|オーナ/アノテーションユーザ|\n|statistics|summarize_task_count_by_task_id|task_idのプレフィックスごとに、タスク数を出力します。|オーナ/アノテーションユーザ|\n|statistics|summarize_task_count_by_user|ユーザごとに担当しているタスク数を出力します。|オーナ/アノテーションユーザ|\n|statistics| visualize             | 統計情報を可視化します。                                                            |オーナ/アノテーションユーザ|\n|supplementary| list             | 補助情報を出力します。                                                           |オーナ/アノテーションユーザ|\n|supplementary| put              | 補助情報を登録します。                                                           |オーナ|\n|task| cancel_acceptance             | 受け入れ完了タスクを、受け入れ取り消し状態にします。                                                         |オーナ|\n|task| change_operator             | タスクの担当者を変更します。                                                             |チェッカー/オーナ|\n|task| complete                | タスクを完了状態にして次のフェーズに進めます（教師付の提出、検査/受入の合格）。                                  |チェッカー/オーナ|\n|task| delete                | タスクを削除します。                                 |オーナ|\n|task|list             | タスク一覧を出力します。                                                            |-|\n|task|list_added_task_history             | タスク履歴情報を加えたタスク一覧を出力します。|オーナ/アノテーションユーザ|\n|task|list_task_history             | タスク履歴の一覧を出力します。|-|\n|task| list_with_json | タスク全件ファイルから一覧を出力します。                               |-|\n|task| put                | タスクを作成します。                                 |オーナ|\n|task| reject                  | タスクを強制的に差し戻します。                                                                 |オーナ|\n|task| update_metadata                  | タスクのメタデータを更新します。                                                                |オーナ/アノテーションユーザ|\n|task_history| list_with_json | タスク履歴全件ファイルから一覧を出力します。                               |-|\n\n# Usage\n\n\n## すべてのコマンドで共通のオプション引数\n\n### `--disable_log`\nログを無効化にします。\n\n\n### `--endpoint_url`\nAnnoFab WebAPIのエンドポイントURLを指定します。デフォルトは https://annofab.com です。\n\n\n### `-h` / `--help`\nコマンドのヘルプを出力します。\n\n```\n# annofabcli全体のヘルプ\n$ annofabcli -h\n\n# project diff コマンドのヘルプ\n$ annofabcli project diff -h\n```\n\n\n### `--logdir`\nログファイルを保存するディレクトリを指定します。指定しない場合、`.log`ディレクトリにログファイルを出力します。\n\n\n### `--logging_yaml`\n以下のような、ロギグングの設定ファイル(YAML)を指定します。指定した場合、`--logdir`オプションは無視されます。指定しない場合、デフォルトのロギング設定ファイルが読み込まれます。\n設定ファイルの書き方は https://docs.python.org/ja/3/howto/logging.html を参照してください。\n\n```yaml:logging-sample.yaml\n# WARNINGレベル以上のログをコンソールに出力する\n\nversion: 1\nhandlers:\n  consoleHandler:\n    class: logging.StreamHandler\nroot:\n  level: WARNING\n  handlers: [consoleHandler]\n\n# デフォルトのロガーを無効化しないようにする https://docs.djangoproject.com/ja/2.1/topics/logging/#configuring-logging\ndisable_existing_loggers: False\n```\n\n\n### `--yes`\n処理中に現れる問い合わせに対して、常に\'yes\'と回答します。\n\n\n## ほとんどのコマンドで共通のオプション引数\n\n\n### `--csv_format`\nCSVのフォーマットをJSON形式で指定します。`--format`が`csv`でないときは、このオプションは無視されます。\n先頭に`file://`を付けると、JSON形式のファイルを指定できます。\n指定した値は、[pandas.DataFrame.to_csv](https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.DataFrame.to_csv.html) の引数として渡されます。\nデフォルトはカンマ区切り、BOM付きUTF-8エンコーディングです。\n\n```\n--csv_format \'{"sep": "\\t"}\'\n```\n\n\n### `-f` / `--format`\nlist系のコマンドで、出力フォーマットを指定します。多くのコマンドでは、以下のフォーマットが指定できます。\n* `csv` : CSV(デフォルとはカンマ区切り)\n* `json` : インデントや空白がないJSON\n* `pretty_json` : インデントされたJSON\n\n\n\n\n\n### `-o` / `--output`\n出力先のファイルパスを指定します。指定しない場合は、標準出力に出力されます。\n主にlist系のコマンドで利用できます。\n\n### `--parallelism`\n並列数を指定します。指定しない場合は、逐次的に処理します。\n\n\n### `-p` / `--project_id`\n対象のプロジェクトのproject_idを指定します。\n\n### `-q` / `--query`\n[JMESPath](http://jmespath.org/) を指定します。出力結果の抽出や、出力内容の変更に利用できます。\n\n\n### `-t` / `--task_id`\n対象のタスクのtask_idを指定します。`file://`を先頭に付けると、task_idの一覧が記載されたファイルを指定できます。\n\n* 相対パスで指定： `--task_id file://task.txt`\n* 絶対パスで指定： `--task_id file:///tmp/task.txt`\n\n\n## デフォルトのログ設定\n* ログは、標準エラー出力とログファイルに出力されます。\n* カレントディレクトリの`.log`ディレクトリに、`annofabcli.log`というログファイルが生成されます。\n* `annofabcli.log`ファイルは、1日ごとにログロテート（新しいログファイルが生成）されます\n\nデフォルトログは https://github.com/kurusugawa-computer/annofab-cli/blob/master/annofabcli/data/logging.yaml で定義されています。\n\n\n## よくある使い方\n\n### 受入完了状態のタスクを差し戻す\n"car"ラベルの"occluded"属性のアノテーションルールに間違いがあったため、以下の条件を満たすタスクを一括で差し戻します。\n* "car"ラベルの"occluded"チェックボックスがONのアノテーションが、タスクに1つ以上存在する。\n\n前提条件\n* プロジェクトのオーナが、annofabcliコマンドを実行する\n\n\n```\n# 受入完了のタスクのtask_id一覧を、acceptance_complete_task_id.txtに出力する。\n$ annofabcli task list --project_id prj1  --task_query \'{"status": "complete","phase":"acceptance"}\' \\\n --format task_id_list --output acceptance_complete_task_id.txt\n\n# 受入完了タスクの中で、 "car"ラベルの"occluded"チェックボックスがONのアノテーションの個数を出力する。\n$ annofabcli annotation list_count --project_id prj1 --task_id file://task.txt --output annotation_count.csv \\\n --annotation_query \'{"label_name_en": "car", "attributes":[{"additional_data_definition_name_en": "occluded", "flag": true}]}\'\n\n# annotation_count.csvを表計算ソフトで開き、アノテーションの個数が1個以上のタスクのtask_id一覧を、task_id.txtに保存する。\n\n# task_id.txtに記載されたタスクを差し戻す。検査コメントは「carラベルのoccluded属性を見直してください」。\n# 差し戻したタスクには、最後のannotation phaseを担当したユーザを割り当てる（画面と同じ動き）。\n$ annofabcli task reject --project_id prj1 --task_id file://tasks.txt --cancel_acceptance \\\n  --comment "carラベルのoccluded属性を見直してください"\n\n```\n\n### プロジェクトメンバをCSVで管理する\n\n```\n# prj1のプロジェクトメンバをCSVで出力する\n$ annofabcli project_member list --project_id prj1 --format csv --output members.csv \\\n --csv_format \'{"columns": ["user_id","member_role","sampling_inspection_rate","sampling_acceptance_rate"],"header":false}\' \n\n\n# members.csvの中身を確認\n$ head members.csv\nuser1,worker\nuser2,accepter\n...\n\n\n# members.csvに記載れたメンバを prj2に登録する\n$ annofabcli project_member put --project_id prj2 --csv members.csv\n\n```\n\n\n## コマンド一覧\n\n### annotation change_attributes\nアノテーションの属性を一括で変更します。ただし、作業中状態のタスクのアノテーションの属性は変更できません。間違えてアノテーション属性を変更したときに復元できるようにするため、`--backup`でバックアップ用のディレクトリを指定することを推奨します。\n\n```\n# task.txtに記載されたタスクのアノテーションの属性をを変更する\n# carラベルのoccludedチェックボックスがTRUEのアノテーションに対して、occludedチェックボックスをOFFにする\n$ annofabcli annotation change_attributes --project_id prj1 --task_id file://task.txt \\ \n--annotation_query \'{"label_name_en": "car", "attributes":[{"additional_data_definition_name_en": "occluded", "flag": true}]}\' \\\n--attributes \'[{"additional_data_definition_name_en": "occluded", "flag": false}]\' \\\n--backup backup_dir\n\n\n```\n\n\n\n\n### annotation delete\nタスク配下のアノテーションを削除します。ただし、作業中/完了状態のタスク、または「過去に割り当てられていて現在の担当者が自分自身でない」タスクのアノテーションは削除できません。\n\n間違えてアノテーションを削除してしまっときに復元できるようにするため、`--backup`でバックアップ用のディレクトリを指定することを推奨します。バックアップ用ディレクトリには、 [annotation dump](#annotation-dump) コマンドの出力結果と同等のアノテーション情報が保存されます。\nアノテーションの復元には [annotation restore](#annotation-restore) コマンドを使用してください。\n\n\n```\n# task.txtに記載されたタスクのアノテーションを削除します。削除する前のアノテーション情報は、`backup`ディレクトリに保存します。\n$ annofabcli annotation delete --project_id prj1 --task_id file://task.txt --backup backup\n\n# carラベルのoccludedチェックボックスがTRUEのアノテーションを削除します\n$ annofabcli annotation delete --project_id prj1 --task_id file://task.txt \\ \n--annotation_query \'{"label_name_en": "car", "attributes":[{"additional_data_definition_name_en": "occluded", "flag": true}]}\' \\\n--backup backup_dir\n```\n\n### annotation dump\n指定したタスク配下のアノテーション情報をディレクトリに保存します。アノテーションをバックアップしたいときなどに利用できます。\nアノテーションの復元には [annotation restore](#annotation-restore) コマンドを使用してください。\n\n\n```\n# task.txtに記載されたタスクのアノテーションを、`output`ディレクトリに保存します。\n$ annofabcli annotation dump --project_id prj1 --task_id file://task.txt --output backup-dir\n```\n\nバックアップディレクトリは、以下のディレクトリ構成です（Simpleアノテーション(v2)と同じディレクトリ構成）。\n`{input_data_id}.json`のフォーマットは、[getEditorAnnotation](https://annofab.com/docs/api/#operation/getEditorAnnotation) APIのレスポンスと同じです。\n\n```\nルートディレクトリ/\n├── {task_id}/\n│\xa0\xa0 ├── {input_data_id}.json\n│\xa0\xa0 ├── {input_data_id}/\n│\xa0\xa0        ├── {annotation_id}............ 塗りつぶしPNG画像\n```\n\n\n\n### annotation import\nアノテーションをプロジェクトにインポートします。\nアノテーションのフォーマットは、Simpleアノテーション(v2)と同じフォルダ構成のzipファイルまたはディレクトリです。\n\n```\nルートディレクトリ/\n├── {task_id}/\n│\xa0\xa0 ├── {input_data_id}.json\n│\xa0\xa0 ├── {input_data_id}/\n│\xa0\xa0        ├── {annotation_id}............ 塗りつぶしPNG画像\n```\n\nJSONフォーマットのサンプルをは以下の通りです。SimpleアノテーションのJSONフォーマットに対応しています。詳しくは https://annofab.com/docs/api/#section/Simple-Annotation-ZIP を参照してください。\n\n\n```json\n{\n    "details": [\n        {\n            "label": "car",\n            "data": {\n                "left_top": {\n                    "x": 878,\n                    "y": 566\n                },\n                "right_bottom": {\n                    "x": 1065,\n                    "y": 701\n                },\n                "_type": "BoundingBox"\n            },\n            "attributes": {}\n        },\n        {\n            "label": "road",\n            "data": {\n                "data_uri": "b803193f-827f-4755-8228-e2c67d0786d9",\n                "_type": "SegmentationV2"\n            },\n            "attributes": {}\n        },\n        {\n            "label": "weather",\n            "data": {\n                "_type": "Classification"\n            },\n            "attributes": {\n                "sunny": true\n            }\n        }\n    ]\n}\n```\n\n以下のように`annotation_id`が指定されている場合、`annotation_id`もインポートされます。\n\n\n```json\n{\n    "details": [\n        {\n            "label": "car",\n            "annotation_id": "12345678-abcd-1234-abcd-1234abcd5678",\n            "data": {\n                "left_top": {\n                    "x": 878,\n                    "y": 566\n                },\n                "right_bottom": {\n                    "x": 1065,\n                    "y": 701\n                },\n                "_type": "BoundingBox"\n            },\n            "attributes": {}\n        },\n\n```\n\nアノテーションをインポートするには、事前に入力データ、タスク、アノテーション仕様を作成する必要があります。\n\nタスクの状態が作業中/完了の場合はインポートしません。\n\n\n```\n# prj1にアノテーションをインポートします。すでにアノテーションが登録されてる場合はスキップします。\n$ annofabcli annotation import --project_id prj1 --annotation simple-annotation.zip \n\n# prj1にアノテーションをインポートします。すでに存在するアノテーションを上書きます。\n$ annofabcli annotation import --project_id prj1 --annotation simple-annotation.zip --overwrite\n```\n\n### annotation list_count\n`task_id`または`input_data_id`で集約したアノテーションの個数を、CSV形式で出力します。\nクエリのフォーマットは、[getAnnotationList API](https://annofab.com/docs/api/#operation/getAnnotationList)のクエリパラメータの`query`キー配下と同じです。\n`label_name_en`(label_idに対応), `additional_data_definition_name_en`(additional_data_definition_idに対応) キーも指定できます。\n\n\n```\n# car ラベルのアノテーション個数を出力する(task_idで集約)\n$ annofabcli annotation list_count --project_id prj1 \\\n --annotation_query \'{"label_name_en": "car"}\'\n\n# car ラベルのアノテーション個数を出力する(input_data_idで集約)\n$ annofabcli annotation list_count --project_id prj1 \\\n --annotation_query \'{"label_name_en": "car"}\' --gropu_by input_data_id\n\n# task.txtに記載されたtask_idの一覧から、car ラベルのアノテーション個数を出力する\n$ annofabcli annotation list_count --project_id prj1 \\\n --annotation_query \'{"label_name_en": "car"}\'  --task_id file://task.txt\n\n# carラベルの"occluded"チェックボックスがONのアノテーションの個数を出力する\n$ annofabcli annotation list_count --project_id prj1 \\\n --annotation_query \'{"label_name_en": "car", "attributes":[{"additional_data_definition_name_en": "occluded", "flag": true}]}\'\n\n# carラベルの"type"ラジオボタン/セレクトボックスが"bus"であるアノテーションの個数を出力する\n$ annofabcli annotation list_count --project_id prj1 \\\n --annotation_query \'{"label_name_en": "car", "attributes":[{"additional_data_definition_name_en": "occluded", "choice_name_en": "bus"}]}\'\n```\n\n#### task_idで集約したときの出力結果（CSV）\n\n| task_id    | annotation_count |\n|------------|------------------|\n| sample_030 | 1                |\n| sample_088 | 2                |\n\n\n#### input_data_idで集約したときの出力結果（CSV）\n\n| task_id    | input_data_id                        | annotation_count |\n|------------|--------------------------------------|------------------|\n| sample_030 | 5738d502-b0a0-4a82-9367-cceffd73cf57 | 1                |\n| sample_093 | dd82cf3a-a38c-4a04-91e7-a4f1ce9af585 | 2                |\n\n\n### annotation restore\n[annotation dump](#annotation-dump) コマンドの出力結果から、アノテーション情報をリストアします。ただし、作業中/完了状態のタスク、または「過去に割り当てられていて現在の担当者が自分自身でない」タスクはリストアできません。\n\n\n```\n# `backup`ディレクトリに保存したアノテーション情報を、`prj1`プロジェクトにリストアする。\n$annofabcli annotation restore --project_id prj1 --task_id file://task.txt --annotation backup-dir\n\n# `backup`ディレクトリに保存したアノテーション情報の内、`task.txt`に記載されたタスクのアノテーション情報を、`prj1`プロジェクトにリストアする。\n$ annofabcli annotation restore --project_id prj1 --task_id file://task.txt --annotation backup-dir\n```\n\n\n\n### annotation_specs history\nアノテーション仕様の履歴一覧を出力します。\n\n```\n# prj1のアノテーション仕様の履歴一覧を出力する\n$ annofabcli annotation_specs history --project_id prj1 \n```\n\n\n\n### annotation_specs list_label\nアノテーション仕様のラベル情報を出力します。\n\n```\n# prj1のアノテーション仕様のラベル情報を、人間が見やすい形式（`--format text`）で出力する\n$ annofabcli annotation_specs list_label --project_id prj1\n\n# prj1のアノテーション仕様のラベル情報を、インデントされたJSONで出力する。\n$ annofabcli annotation_specs list_label --project_id prj1 --format pretty_json\n\n# 最新より１つ前の履歴である、アノテーション仕様を出力する。\n$ annofabcli annotation_specs list_label --project_id prj1 --before 1\n\n# history_idがXXXのアノテーション仕様を出力する。（history_idは、`annofabcli annotation_specs history`コマンドで取得する）\n$ annofabcli annotation_specs list_label --project_id prj1 --history_id XXX\n\n\n```\n\n#### `--format text`の出力結果 \n`--format text`の出力結果は、以下のフォーマットで出力されます。\n\n```\nlabel_id    label_type    label_name_ja    label_name_en\n    attribute_id    attribute_type    attribute_name_ja    attribute_name_ja\n        choice_id    choice_name_ja    choice_name_en\n        ...\n    ...\n...\n```\n\nサンプルプロジェクトでコマンドを実行した結果は、以下の通りです。\n\n```\n15ba7932-24b9-4cf3-95bd-9bf6deede4fa\tbounding_box\tネコ\tCat\n\te6864d96-78fa-45f3-a786-6c8c900c92ae\tflag\t隠れ\toccluded\n\t51e8c91f-5de1-450b-a0f3-94fec582f5ce\tlink\t目のリンク\tlink-eye\n\taff2855e-2e3d-47a2-8c27-c7652e4dfb2f\tinteger\t体重\tweight\n\t7e6a577a-3410-4c8a-9624-2904bb2e6666\tcomment\t名前\tname\n\ta63a0513-a96e-4c7c-8754-88a24fef9ca9\ttext\t備考\tmemo\n\t649abf45-1ed7-459a-8282-a58228e9a302\ttracking\tobject id\tobject id\nc754f724-5f8c-48eb-81ec-ea77e55efee7\tpolyline\t足\tleg\nf50aa88d-36c7-43f5-8728-247a49b4f4d8\tpoint\t目\teye\n108ce1f7-217b-43e9-a407-8d0ac6aad87e\tsegmentation\t犬\tdog\n2ffb4c74-106b-44ac-81ce-3c3df77518e0\tsegmentation_v2\t人間\thuman\nded52dcb-bcd6-4e77-9626-61e546f635d0\tpolygon\t鳥\tbird\n5ac0d7d5-6738-4c4b-a69a-cd583ff458e1\tclassification\t気候\tclimatic\n\t896d7eeb-9c60-4fbf-b7c4-8f4209261049\tchoice\t天気\tweather\n\t\tc9615782-b872-4641-9be4-0fb4f905d966\t\t晴\tsunny\n\t\t553018a5-e594-4536-bc05-876fa6b48ed5\t\t雨\trainy\n\t60caffa5-6300-4819-9a99-c43ce49008c2\tselect\t気温\ttemparature\n\t\t89b3577d-a245-4b85-82ef-6569ecbf8ad7\t\t10\t10\n\t\tbdcd4d5b-cecc-4ec9-9038-d9284cd4f475\t\t20\t20\n\t\t9f3a0355-2cc8-412a-9129-3b62fa7b6ead\t\t30\t30\n\t\t2726336c-96d3-485b-9f96-7d4bcc97083b\t\t40\t40\n\n```\n\n\n### annotation_specs list_label_color\nアノテーション仕様から、label_name(english)とRGBを対応付けたJSONを出力します。\n\n```\n$ annofabcli annotation_specs list_label_color --project_id prj1 \n```\n\n以下のJSONのような出力結果になります。\n\n```json:出力結果\n{\n  "cat": [\n    255,\n    99,\n    71\n  ],\n  "dog": [\n    255,\n    0,\n    255\n  ],\n```\n\n\n\n\n### filesystem write_annotation_image\nアノテーションzip、またはそれを展開したディレクトリから、アノテーションの画像（Semantic Segmentation用）を生成します。\n以下のアノテーションが画像化対象です。\n* 矩形\n* ポリゴン\n* 塗りつぶし\n* 塗りつぶしv2\n\n\n```\n# アノテーションzipをダウンロードする。\n$ annofabcli project download simple_annotation --project_id prj1 --output annotation.zip\n\n\n# label_nameとRGBを対応付けたファイルを生成する\n$ annofabcli annotation_specs list_label_color --project_id prj1 --output label_color.json\n\n\n# annotation.zip から、アノテーション画像を生成する\n$ annofabcli filesystem write_annotation_image  --annotation annotation.zip \\\n --image_size 1280x720 \\\n --label_color file://label_color.json \\\n --output_dir /tmp/output\n\n\n# annotation.zip から、アノテーション画像を生成する。ただしタスクのステータスが"完了"で、task.txtに記載れたタスクのみ画像化する。\n$ annofabcli filesystem write_annotation_image  --annotation annotation.zip \\\n --image_size 1280x720 \\\n --label_color file://label_color.json \\\n --output_dir /tmp/output \\\n --task_status_complete \\\n --task_id file://task.txt\n\n\n# 入力データのメタデータ"width", "height"に設定した画像サイズを参照して、アノテーション画像を生成する\n$ annfoabcli project download input_data --project_id prj1 --output input_data.json\n$ annofabcli filesystem write_annotation_image  --annotation annotation.zip \\\n --input_data_json input_data.json \\\n --metadata_key_of_image_size width height \\\n --label_color file://label_color.json \\\n --output_dir /tmp/output\n```\n\n#### 出力結果（塗りつぶし画像）\n\n![filesystem write_annotation_iamgeの塗りつぶし画像](readme-img/write_annotation_image-output.png)\n\n\n### input_data delete\nタスクに使われていない入力データを削除します。\n\n```\n# 入力データ input1, input2 を削除する\n$ annofabcli input_data delete --project_id prj1 --input_data_id input1 input2\n\n# `input_data_id.txt` ファイルに記載されている入力データを削除する\n$ annofabcli input_data delete --project_id prj1 --input_data_id file://input_data_id.txt\n```\n\n\n### input_data list\n入力データ一覧を出力します。\n\n```\n# input_data_nameが"sample"の入力データ一覧を出力する\n$ annofabcli input_data list --project_id prj1 --input_data_query \'{"input_data_name": "sample"}\' \n\n# input_data_idが"id1", "id2"の入力データを取得する\n$ annofabcli input_data list --project_id prj1 --input_data_id id1 id2\n\n# 入力データの詳細情報（参照されているタスクのtask_id `parent_task_id_list`）も出力する\n$ annofabcli input_data list --project_id prj1 --input_data_query \'{"input_data_name": "sample"}\' --add_details\n\n```\n\n\n#### 出力結果（CSV）\n\n\n| etag                             | input_data_id                        | input_data_name                   | input_data_path                                                                                                                                                                 | original_input_data_path                                                                                                                                                        | original_resolution | project_id                           | resized_resolution | sign_required | task_id_list   | updated_datetime              | url                                                                                                                                                                        |\n|----------------------------------|--------------------------------------|-----------------------------------|---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|---------------------|--------------------------------------|--------------------|---------------|----------------|-------------------------------|----------------------------------------------------------------------------------------------------------------------------------------------------------------------------|\n| a43717273502b67a1989c9b25e252cde | 3c8d8f15-14f0-467a-a8fe-562cbbccf08a | val.zip/val/9a70bdec-1504e338.jpg | s3://example.com/example | s3://example.com/example |                     | 58a2a621-7d4b-41e7-927b-cdc570c1114a |                    | False         | [\'sample_247\'] | 2019-04-19T16:36:17.846+09:00 | https://annofab.com/projects/example/input_data/example |\n\n\n\n\n\n### input_data list_merged_task\nタスク一覧と結合した入力データ一覧のCSVを出力します。\n動画プロジェクトで、各タスクの動画時間を調べる際などに利用できます。\n\n```\n# prj1の入力データ全件ファイル、タスク全件ファイルをダウンロードして、マージしたCSVを出力する\n$ annofabcli input_data list_merged_task --project_id prj1 --output input_data.csv\n\n# prj1の入力データ全件ファイル、タスク全件ファイルの最新版をダウンロードして、マージしたCSVを出力する\n$ annofabcli input_data list_merged_task --project_id prj1 --output input_data.csv --latest\n\n# 入力データ全件ファイル、タスク全件ファイルのJSONから、マージしたCSVを出力する\n$ annofabcli input_data list_merged_task --input_data_json input_data.json --task_json task.json --output input_data.csv\n```\n\n\n### input_data list_with_json\n入力データ全件ファイルから一覧を出力します。\n\n```\n# 全件の入力データを出力する\n$ annofabcli input_data list_with_json --project_id prj1 --output input_data.csv\n\n# 入力データ全件ファイルを最新化してから、出力する\n$ annofabcli input_data list_with_json --project_id prj1 --output input_data.csv --latest\n```\n\n\n\n### input_data put\nCSVに記載された入力データ情報やzipファイルを、入力データとして登録します。\n\n#### CSVに記載された入力データ情報を、入力データとして登録\n\n* ヘッダ行なし\n* カンマ区切り\n* 1列目: input_data_name. 必須\n* 2列目: input_data_path. 必須. 先頭が`file://`の場合、ローカルのファイルを入力データとしてアップロードします。\n* 3列目: input_data_id. 省略可能。省略した場合UUIDv4になる。\n* 4列目: sign_required. 省略可能. `true` or `false`\n\nCSVのサンプル（`input_data.csv`）です。\n\n```\ndata1,s3://example.com/data1,id1,\ndata2,s3://example.com/data2,id2,true\ndata3,s3://example.com/data3,id3,false\ndata4,https://example.com/data4,,\ndata5,file://sample.jpg,,\ndata6,file:///tmp/sample.jpg,,\n```\n\n\n```\n# input_data.csvに記載されている入力データを登録する。すでに入力データが存在する場合はスキップする。\n$ annofabcli input_data put --project_id prj1 --csv input_data.csv\n\n# input_data.csvに記載されている入力データを登録する。すでに入力データが存在する場合は上書きする。\n$ annofabcli input_data put --project_id prj1 --csv input_data.csv --overwrite\n\n# input_data.csvに記載されている入力データを、並列処理で登録する（`--yes`オプションが必要）。\n$ annofabcli input_data put --project_id prj1 --csv input_data.csv --parallelism 2 --yes\n```\n\n\n`input_data list`コマンドを使えば、プロジェクトに既に登録されている入力データからCSVを作成できます。\n\n```\n$ annofabcli input_data list --project_id prj1 --input_data_query \'{"input_data_name": "sample"}\'  \\\n --format csv --output input_data.csv \\\n --csv_format \'{"columns": ["input_data_name","input_data_path", "input_data_id", "sign_required"], "header":false}\' \n```\n\n\n#### zipファイルを入力データとして登録\n\n\n```\n# 画像や動画が格納されたinput_data.zipを、入力データとして登録する\n$ annofabcli input_data put --project_id prj1 --zip input_data.zip\n\n# zipファイルを入力データとして登録し、入力データの登録が完了するまで待つ。\n$ annofabcli input_data put --project_id prj1 --zip input_data.zip --wait\n\n# zipファイルを入力データとして登録する。そのときinput_data_nameを`foo.zip`にする。\n$ annofabcli input_data put --project_id prj1 --zip input_data.zip --input_data_name_for_zip foo.zip\n\n```\n\n\n\n### input_data update_metadata\n入力データのメタデータを更新します。\n\n```\n# `input_data.txt`に記載されている入力データIDに対して、入力データのメタデータに\'{"foo":"bar"}\' を設定します。\n$ annofabcli input_data update_metadata --project_id prj1 --input_data_id --file://input_data.txt \\\n --metadata \'{"foo":"bar"}\'\n\n# `input_data.txt`に記載されている入力データIDに対して、入力データのメタデータを \'{"foo":"bar"}\' に変更します。\n# すでに設定されているメタデータは上書きされます\n$ annofabcli input_data update_metadata --project_id prj1 --input_data_id --file://input_data.txt \\\n --metadata \'{"foo":"bar"}\'  --overwrite\n```\n\n\n### inspection_comment list\n検査コメント一覧を出力します。\n\n```\n# task1, task2の検査コメント一覧を、CSVで出力する\n$ annofabcli inspection_comment list --project_id prj1 --task_id task1 task2\n\n# タブ区切りの"out.tsv"を出力する\n$ annofabcli inspection_comment list --project_id prj1 --task_id task1 task2 \\\n --format csv --csv_format \'{"sep":"\\t"}\'  --output out.tsv\n\n# JSONで出力する\n$ annofabcli inspection_comment list --project_id prj1 --task_id file://task.txt --format json\n\n# 検査コメント情報が記載されたファイルを元にして、検査コメント一覧を出力します\n# 検査コメント情報が記載されたファイルは、`$ annofabcli project download inspection_comment`コマンドで取得できます。\n$ annofabcli inspection_comment list --project_id prj1 --inspection_comment_json inspection_comment.json\n\n# 返信コメントを除外した検査コメント一覧を出力します\n$ annofabcli inspection_comment list --project_id prj1 --task_id task1 task2 --exclude_reply\n\n# 返信コメントのみの一覧を出力します\n$ annofabcli inspection_comment list --project_id prj1 --task_id task1 task2 --only_reply\n\n\n```\n\n#### 出力結果（CSV）\n\n| project_id                           | task_id    | input_data_id                        | inspection_id                        | phase      | phase_stage | commenter_account_id                 | annotation_id                        | label_id                             | data                                  | parent_inspection_id | phrases | comment | status          | created_datetime              | updated_datetime              | commenter_user_id | commenter_username | phrase_names_en | phrase_names_ja | label_name_en | label_name_ja | input_data_index |\n|--------------------------------------|------------|--------------------------------------|--------------------------------------|------------|-------------|--------------------------------------|--------------------------------------|--------------------------------------|---------------------------------------|----------------------|---------|---------|-----------------|-------------------------------|-------------------------------|-------------------|--------------------|-----------------|-----------------|---------------|---------------|------------------|\n| 58a2a621-7d4b-41e7-927b-cdc570c1114a | sample_180 | bf6b4790-cdb8-4d4d-85bb-08550934ed61 | 5f096677-67e4-4e75-9eac-bbd8ac9694d9 | inspection | 1           | 12345678-abcd-1234-abcd-1234abcd5678 | 8aff181e-9df4-4c66-8fb2-10596c686d5c | 8aff181e-9df4-4c66-8fb2-10596c686d5c | {\'x\': 358, \'y\': 48, \'_type\': \'Point\'} |                      | []      | 枠がずれています     | error_corrected | 2019-07-26T17:41:16.626+09:00 | 2019-08-01T10:57:45.639+09:00 | user_id   | username          | []              | []              | car           | car           | 0                |\n\n\n\n### inspection_comment list_with_json\n検査コメント一覧を出力します。\n\n```\n# 検査コメント全件を出力する\n$ annofabcli inspection_comment list_with_json --project_id prj1 --output inspection_comment.csv\n\n```\n\n\n\n### inspection_comment list_unprocessed\n未処置の検査コメント一覧を出力します。\n\n```\n# 未処置の検査コメント一覧を出力する\n$ annofabcli inspection_comment list_unprocessed --project_id prj1 --task_id file://task.txt\n\n# 未処置で、user1が"hoge"とコメントした検査コメント一覧を出力する\n$ annofabcli inspection_comment list_unprocessed  --project_id prj1 --task_id file://task.txt \\\n --inspection_comment "hoge" --commenter_user_id user1 --format pretty_json --output inspection.json\n\n# 検査コメント情報が記載されたファイルを元にして、検査コメント一覧を追加します\n$ annofabcli inspection_comment list_unprocessed --project_id prj1 --inspection_comment_json inspection_comment.json\n```\n\n\n### instruction copy\n作業ガイドを別のプロジェクトにコピーします。\n\n\n```\n# prj1の作業ガイドをprj2にコピーする\n$ annofabcli instruction copy prj1 prj2\n```\n\n\n\n### instruction upload\nHTMLファイルを作業ガイドとして登録します。\nimg要素のsrc属性がローカルの画像を参照している場合（http, https, dataスキーマが付与されていない）、画像もアップロードします。\n\n`instruction.html`の中身。\n\n```html\n<html>\n<head></head>\n<body>\n作業ガイドのサンプル\n<img src="lena.png">\n</body>\n</html>\n```\n\n```\n$ annofabcli instruction upload --project_id prj1 --html instruction.html\n```\n\n\n#### Confluenceのページを作業ガイド用にHTMLとして保存する場合\n1. Confluenceのエクスポート機能で、ページをエクスポートする。\n    * HTMLファイルと添付画像が含まれたzipファイルをダウンロードする。\n2. エクスポートしたHTMLのスタイルを、style属性に反映させる。AnnoFabの作業ガイドには、スタイルシートを登録できないため。\n    1. エクスポートしたファイルをChromeで開く\n    2. Chrome開発ツールのConfoleタブで以下のJavaScriptを実行して、全要素のborder, color, backgroundスタイルを、style属性に反映させる\n\n        ```js\n        elms = document.querySelectorAll("body *");\n        for (let e of elms) {\n            s = window.getComputedStyle(e);\n            e.style.background = s.background;\n            e.style.color = s.color;\n            e.style.border = s.border;\n        }\n        ```\n    3. Chrome開発ツールのElementタブで、html要素をコピー(Copy outerHTML)して、HTMLファイルを上書きする\n\n\n### job delete\nジョブを削除します。\n削除対象のjob_idは`annofabcli job list`コマンドで確認できます。\n\n```\n# アノテーション更新のジョブを削除します。\n$ annofabcli job delete --project_id prj1 --job_type gen-annotation --job_id 12345678-abcd-1234-abcd-1234abcd5678\n```\n\n\n### job list\nジョブ一覧を出力します。\n\n```\n# アノテーション更新のジョブ一覧を取得します（最新のジョブ1個のみ）\n$ annofabcli job list --project_id prj1 --job_type gen-annotation\n\n# タスク作成のジョブ一覧を取得します（最大200個）\n$ annofabcli job list --project_id prj1 --job_type gen-tasks --job_query \'{"limit": 200}\'\n\n```\n\n\n\n### job list_last\n複数のプロジェクトに対して、最新のジョブを出力します。\n\n```\n# prj1, prj2に対して、「アノテーション更新」のジョブを出力します。\n$ annofabcli job list_last --project_id prj1 prj2 --job_type gen-annotation\n\n# 組織 org1配下のプロジェクト（進行中で、自分自身が所属している）に対して、「タスク全件ファイル更新」のジョブを出力します。\n$ annofabcli job list_last --organization org1 --job_type gen-tasks-list\n\n# アノテーションの最終更新日時を、タスクの最終更新日時と比較して出力します。\n$ annofabcli job list_last --project_id prj1 --job_type gen-annotation --add_details \\\n --csv_format \'{"columns": ["project_id","project_title","job_status","updated_datetime", "task_last_updated_datetime"]}\' \n\n```\n\n\n### job wait\nジョブの終了を待ちます。\n\n```\n# アノテーション更新のジョブが終了するまで待ちます。\n$ annofabcli job wait --project_id prj1 --job_type gen-annotation \n```\n\n\n### labor list_worktime_by_user\n\nユーザごとに作業予定時間、作業実績時間を出力します。\n\n```\n# 組織org1, org2に対して、user1, user2の作業時間を集計します。\n$ annofabcli labor list_worktime_by_user --organization org1 org2 --user_id user1 user2 \\\n --start_date 2019-10-01 --end_date 2019-10-31 --output_dir /tmp/output\n\n# プロジェクトprj1, prj2に対して作業時間を集計します。集計対象のユーザはプロジェクトに所属するメンバです。\n$ annofabcli labor list_worktime_by_user --project_id prj1 prj2 --user_id user1 user2 \\\n --start_date 2019-10-01 --end_date 2019-10-31 --output_dir /tmp/output\n\n\n# user.txtに記載されているユーザの予定稼働時間も一緒に出力します。\n$ annofabcli labor list_worktime_by_user --project_id prj1 prj2 --user_id file://user.txt \\\n --start_month 2019-10 --end_month 2019-11 --add_availability --output_dir /tmp/output\n\n```\n\n\n### organization_member list\n組織メンバ一覧を出力します。\n\n```\n# 組織org1の組織メンバ一覧を出力します。\n$ annofabcli organization_member list --organization org1\n\n```\n\n\n\n### project change_status\nプロジェクトのステータスを変更します。\n\n\n```\n# prj1, prj2を停止状態にします。\n$ annofabcli project change_status --project_id prj1 prj2 --status suspended\n\n\n# prj1, prj2を進行状態にします。\n$ annofabcli project change_status --project_id prj1 prj2 --status active\n```\n\n\n\n### project cooy\nプロジェクトをコピーして（アノテーション仕様やメンバーを引き継いで）、新しいプロジェクトを作成します。\n\n\n\n```\n# prj1 プロジェクトをコピーして、"prj2-title"というプロジェクトを作成する\n$ annofabcli project copy --project_id prj1 --dest_title "prj2-title"\n\n\n# prj1 プロジェクトをコピーして、"prj2"というプロジェクトIDのプロジェクトを作成する。\n# コピーが完了するまで待つ(処理を継続する)\n$ annofabcli project copy --project_id prj1 --dest_title "prj2-title" --dest_project_id prj2 \\\n --wait\n\n\n# prj1 プロジェクトの入力データと、タスクをコピーして、"prj2-title"というプロジェクトを作成する\n$ annofabcli project copy --project_id prj1 --dest_title "prj2-title" --copy_inputs --copy_tasks\n\n\n```\n\n\n\n\n### project diff\nプロジェクト間の差分を、以下の項目について表示します。差分がない場合、標準出力は空になります。\n* アノテーション仕様のラベル情報\n* 定型指摘\n* プロジェクトメンバ\n* プロジェクトの設定\n\n\n```\n# すべての差分\n$ annofabcli project diff  prj1 prj2\n\n# アノテーション仕様のラベル情報の差分\n$ annofabcli project diff prj1 prj2 --target annotation_labels\n\n# 定型指摘の差分\n$ annofabcli project diff prj1 prj2 --target inspection_phrases\n\n# プロジェクトメンバの差分\n$ annofabcli project diff  prj1 prj2 --target members\n\n# プロジェクト設定の差分\n$ annofabcli project diff  prj1 prj2 --target settings\n\n```\n\n\n\nプロジェクト間の差分は、以下のように出力されます。\n`dict`型の差分は、[dictdiffer](https://dictdiffer.readthedocs.io/en/latest/)のフォーマットで出力します。\n\n```\n=== prj1_title1(prj1) と prj1_title2(prj2) の差分を表示\n=== プロジェクトメンバの差分 ===\nプロジェクトメンバは同一\n=== プロジェクト設定の差分 ===\nプロジェクト設定は同一\n=== 定型指摘の差分 ===\n定型指摘は同一\n=== アノテーションラベル情報の差分 ===\nラベル名(en): car は差分あり\n[(\'change\', \'color.red\', (4, 0)),\n (\'change\', \'color.green\', (251, 255)),\n (\'change\', \'color.blue\', (171, 204))]\nラベル名(en): bike は同一\n```\n\n\n\n### project download\nプロジェクトに対して、タスクや検査コメント、アノテーションなどをダウンロードします。\nダウンロード対象は以下の通りです。\n* すべてのタスクが記載されたJSON\n* すべての入力データが記載されたJSON\n* すべての検査コメントが記載されたJSON\n* すべてのタスク履歴イベントが記載されたJSON\n* すべてのタスク履歴イベントが記載されたJSON（非推奨）\n* Simpleアノテーションzip\n* Fullアノテーションzip（非推奨）\n\n\n```\n# タスクの全一覧が記載されたJSONファイルをダウンロードする\n$ annofabcli project download task --project_id prj1 --output task.json\n\n# 入力データのすべてが記載されたJSONファイルをダウンロードする\n$ annofabcli project download input_data --project_id prj1 --output input_data.json\n\n# 検査コメントの全一覧が記載されたJSONファイルをダウンロードする\n$ annofabcli project  download inspection_comment --project_id prj1 --output inspection_comment.json\n\n# タスク履歴イベントの全一覧が記載されたJSONファイルをダウンロードする\n$ annofabcli project download task_history --project_id prj1 --output task_history.json\n\n# Simpleアノテーションのzipファイルをダウンロードする\n$ annofabcli project download simple_annotation --project_id prj1 --output simple_annotation.zip\n\n# 最新のSimpleアノテーションのzipファイルをダウンロードする\n$ annofabcli project download simple_annotation --project_id prj1 --output simple_annotation.zip --latest\n\n# 最新のタスク全一覧が記載されたJSONファイルをダウンロードする\n$ annofabcli project download task --project_id prj1 --output task.json --latest\n\n# アノテーションの最新化を最大60分(60秒間隔で最大60回アクセス)待つ\n$ annofabcli project download simple_annotation --project_id prj1  58a2a621-7d4b-41e7-927b-cdc570c1114a --output simple_annotation.zip --latest \\\n --wait_options \'{"interval":60, "max_tries":60}\' \n```\n\n\n### project list\nプロジェクト一覧を出力します。\n\n```\n# org1配下のプロジェクトで、user1が所属する進行中のプロジェクト一覧を出力する。\n$ annofabcli project list --organization org1 --project_query \'{"status": "active", "user_id": "user1}\'\n\n# prj1, prj2のプロジェクト一覧を出力する\n$ annofabcli project list --project_id prj1 prj2\n```\n\n\n### project update_annotation_zip\nアノテーションzipを更新します。\n\n```\n# prj1, prj2のアノテーションzipを更新します。更新する必要がなければ更新しません。\n$ annofabcli project update_anotation_zip --project_id prj1 prj2\n\n# prj1, prj2のアノテーションzipを更新します。更新する必要がなくても常に更新します。\n$ annofabcli project update_anotation_zip --project_id prj1 prj2 --force\n\n# prj1, prj2のアノテーションzipを更新して、すべてのプロジェクトの更新が完了するまで待ちます\n$ annofabcli project update_anotation_zip --project_id prj1 prj2 --wait\n\n# アノテーションzipの更新が完了するまで待つ処理を並列で実行します。\n$ annofabcli project update_anotation_zip --project_id prj1 prj2 --wait --parallelism 4\n```\n\n\n\n### project_member change\n複数のプロジェクトメンバに対して、メンバ情報を変更します。ただし、自分自身は変更できません。\n\n```\n# user1, user2のロールを"worker"（アノテータ）に変更する\n$ annofabcli project_member change --project_id prj1 --user_id user1 user2 --role worker\n\n# `user_id.txt`に記載されたuser_idに対して、抜取検査率、抜取受入率を指定する\n$ annofabcli project_member change --project_id prj1 --user_id file://user_id.txt \\\n --member_info \'{"sampling_inspection_rate": 10, "sampling_acceptance_rate": 20}\'\n\n# すべてのユーザに対して、抜取検査率を未設定にする\n$ annofabcli project_member change --project_id prj1 --all_user \\\n --member_info \'{"sampling_inspection_rate": null}\'\n\n```\n\n\n\n### project_member copy\nプロジェクトメンバを別のプロジェクトにコピーします。\n\n```\n# prj1のメンバをprj2にコピーする。\n$ annofabcli project_member copy prj1 prj2\n\n# prj1のメンバをprj2にコピーする。prj2にしか存在しないメンバは削除される。\n$ annofabcli project_member copy prj1 prj2 --delete_dest\n```\n\n\n\n### project_member delete\n複数のプロジェクトからユーザを削除します。\n\n```\n# ORG組織配下のすべてのプロジェクトから、user1, user2を削除する\n$ annofabcli project_member delete --user_id user1 user2  --organization ORG\n\n# prj1, prj2のプロジェクトからuser1をaccepterロールで招待する\n$ annofabcli project_member invite --user_id user1  --project_id prj1 prj2\n```\n\n\n\n### project_member invite\n複数のプロジェクトに、ユーザを招待します。\n\n```\n# ORG組織配下のすべてのプロジェクトに、user1, user2をownerロールで招待する\n$ annofabcli project_member invite --user_id user1 user2 --role owner --organization ORG\n\n# prj1, prj2のプロジェクトに、user1をaccepterロールで招待する\n$ annofabcli project_member invite --user_id user1 --role accepter --project_id prj1 prj2\n```\n\n\n### project_member list\nプロジェクトメンバ一覧を出力します。\n\n```\n# ORG組織配下のすべてのプロジェクトのプロジェクトメンバ一覧を出力する\n$ annofabcli project_member list --organization ORG\n\n# prj1, prj2のプロジェクトのプロジェクトメンバ一覧を出力する\n$ annofabcli project_member list --project_id prj1 prj2\n```\n\n#### 出力結果（CSV）\n\n| project_id                           | account_id                           | user_id         | username  | member_status | member_role | updated_datetime              | created_datetime              | sampling_inspection_rate | sampling_acceptance_rate | project_title                |\n|--------------------------------------|--------------------------------------|-----------------|-----------|---------------|-------------|-------------------------------|-------------------------------|--------------------------|--------------------------|------------------------------|\n| 12345678-abcd-1234-abcd-1234abcd5678 | 12345678-abcd-1234-abcd-1234abcd5678 | user_id | username | active        | owner       | 2019-09-10T14:51:00.908+09:00 | 2019-04-19T16:29:41.069+09:00 |                          |                          | sample_project |\n\n\n### project_member put\nCSVに記載されたユーザを、プロジェクトメンバとして登録します。\n\nmembers.csvの中身は以下の通りです。\n\n* ヘッダ行なし\n* カンマ区切り\n* 1列目: user_id. 必須\n* 2列目: member_role. 必須.  `owner`, `worker`, `accepter`, `training_data_user` のいずれか。\n* 3列目: sampling_inspection_rate. 省略可能。\n* 4列目: sampling_acceptance_rate. 省略可能。\n\n\n```\nuser1,worker\nuser2,accepter,80,40\n```\n\n\n```\n# CSVに記載れたユーザを、prj1プロジェクトのメンバとして登録します。\n$ annofabcli project_member put --project_id prj1 --csv members.csv\n\n# CSVに記載れたユーザを、prj1プロジェクトのメンバとして登録します。csvに記載されていないユーザは削除します。\n$ annofabcli project_member put --project_id prj1 --csv members.csv --delete\n```\n\n### statistics list_annotation_count\n各ラベル、各属性値のアノテーション数を、タスクごと/入力データごとに出力します。\n`--annotation`にはAnnoFabからダウンロードしたSimpleアノテーションzipのパスを渡します。指定しない場合はAnnoFabからダウンロードします。\n出力結果には以下のファイルが含まれています。\n* `labels_count.csv`：各ラベルのアノテーション数\n* `attirbutes_count.csv`：各属性値のアノテーション数（ただし属性の種類がチェックボックス、ラジオボタン、セレクトボックスの属性のみが対象）\n\n\n```\n# タスクごとにアノテーション数を、output ディレクトリに出力\n$ annofabcli statistics list_annotation_count --project_id prj1 --output_dir output --annotation annotataion.zip\n\n# 入力データごとにアノテーション数を、output ディレクトリに出力。アノテーション情報はAnnoFabからダウンロードする\n$ annofabcli statistics list_annotation_count --project_id prj1 --output_dir output --group_by input_data_id\n\n```\n\n\n### statistics list_by_date_user\n\nタスク数や作業時間などの情報を、日ごとユーザごとに出力します。\n\n```\n$ annofabcli statistics list_by_date_user --project_id prj1 --output data.csv\n\n```\n\n\n\n### statistics list_cumulative_labor_time\nタスクフェーズ別の累積作業時間をCSV形式で出力します。\n\n```\n$ annofabcli statistics list_cumulative_labor_time --project_id prj1 --output stat.csv\n```\n\n### statistics list_task_progress\nタスク進捗状況をCSV形式で出力します。\n\n```\n$ annofabcli statistics list_task_progress --project_id prj1 --output stat.csv\n```\n\n### statistics merge_visualization_dir\n`annofabcli statistics visualize`コマンドの出力結果をマージします。\n\n\n```\n$ annofabcli statistics visualize --project_id prj1 --output outdir1\n$ annofabcli statistics visualize --project_id prj2 --output outdir2\n$ annofabcli statistics merge_visualization_dir --dir outdir1 outdir2 --output_dir merge_dir\n```\n\n\n### statistics summarize_task_count\nタスクのフェーズ、ステータス、ステップごとにタスク数を、CSV形式で出力します。\n「1回目の教師付」と「2回目の教師付」を区別して集計されます。\n\n\n```\n# prj1のタスク数を出力します。ダウンロードしたタスク全件ファイルを元にして出力します（AM02:00頃更新）。\n$ annofabcli statistics summarize_task_count --project_id prj1 --output task-count.csv\n\n# `annofabcli project download task`でダウンロードした`task.json`を元にして、タスク数を出力します。\n$ annofabcli statistics summarize_task_count --project_id prj1 --task_json task.json --output task-count.csv\n\n```\n\n以下のようなCSVが出力されます。\n\n```csv\nstep,phase,phase_stage,simple_status,task_count\n1,annotation,1,not_started,3761\n1,annotation,1,working_break_hold,30\n1,acceptance,1,not_started,1861\n1,acceptance,1,working_break_hold,20\n2,annotation,1,not_started,225\n2,annotation,1,working_break_hold,3\n2,acceptance,1,not_started,187\n5,acceptance,1,not_started,1\n,acceptance,1,complete,3000\n```\n\n\n* step：何回目のフェーズか\n* simple_status：タスクステータスを簡略化したもの\n    * not_started：未着手\n    * working_break_hold：作業中か休憩中か保留中\n    * complete：完了\n\n「一度も作業されていない教師付未着手」のタスク数は、先頭行（step=1, phase=annotation, simple_status=not_started）のtask_countから分かります。\n\n\n### statistics summarize_task_count_by_task_id\ntask_idのプレフィックスごとに、タスク数をCSV形式で出力します。\ntask_idは`{prefix}_{連番}`のようなフォーマットを想定しています。\n\n\n```\n# prj1のタスク数を出力します。ダウンロードしたタスク全件ファイルを元にして出力します（AM02:00頃更新）。\n$ annofabcli statistics summarize_task_count_by_task_id --project_id prj1 --output task-count.csv\n\n# `annofabcli project download task`でダウンロードした`task.json`を元にして、タスク数を出力します。\n$ annofabcli statistics summarize_task_count_by_task_id --project_id prj1 --task_json task.json --output task-count.csv\n\n```\n\n以下のようなCSVが出力されます。\n\n```csv\ntask_id_prefix,complete,on_hold,annotation_not_started,inspection_not_started,acceptance_not_started,other,sum\n20200401,10,0,0,0,0,0,10\n20200501,10,1,4,0,1,4,20\n```\n\n各列\n* annotation_not_started: 教師付フェーズが一度も作業されていないタスク数\n* inspection_not_started: 検査フェーズが一度も作業されていないタスク数\n* acceptance_not_started: 受入フェーズが一度も作業されていないタスク数\n* other: 休憩中、作業中、\n* simple_status：タスクステータスを簡略化したもの\n    * not_started：未着手\n    * working_break_hold：作業中か休憩中か保留中\n    * complete：完了\n\n「一度も作業されていない教師付未着手」のタスク数は、先頭行（step=1, phase=annotation, simple_status=not_started）のtask_countから分かります。\n\n\n### statistics summarize_task_count_by_user\nユーザごとに担当しているタスク数をCSV形式で出力します。\n\n\n```\n# prj1のタスク数を出力します。ダウンロードしたタスク全件ファイルを元にして出力します（AM02:00頃更新）。\n$ annofabcli statistics summarize_task_count_by_user --project_id prj1 --output task-count.csv\n\n# `annofabcli project download task`でダウンロードした`task.json`を元にして、タスク数を出力します。\n$ annofabcli statistics summarize_task_count_by_task_id --project_id prj1 --task_json task.json --output task-count.csv\n\n```\n\n以下のようなCSVが出力されます。\n\n```csv\ntask_id_prefix,complete,on_hold,annotation_not_started,inspection_not_started,acceptance_not_started,other,sum\n20200401,10,0,0,0,0,0,10\n20200501,10,1,4,0,1,4,20\n```\n\n各列\n* annotation_not_started: 教師付フェーズが一度も作業されていないタスク数\n* inspection_not_started: 検査フェーズが一度も作業されていないタスク数\n* acceptance_not_started: 受入フェーズが一度も作業されていないタスク数\n* other: 休憩中、作業中、\n* simple_status：タスクステータスを簡略化したもの\n    * not_started：未着手\n    * working_break_hold：作業中か休憩中か保留中\n    * complete：完了\n\n「一度も作業されていない教師付未着手」のタスク数は、先頭行（step=1, phase=annotation, simple_status=not_started）のtask_countから分かります。\n\n\n### statistics visualize\n統計情報を可視化します。\n\n```\n# prj1の統計情報を可視化したファイルを、/tmp/outputに出力する\n$ annofabcli statistics visualize --project_id prj1 --output_dir /tmp/output\n\n# statusがcompleteのタスクを統計情報を可視化したファイルを、/tmp/outputに出力する\n$ annofabcli statistics visualize --project_id prj1 --output_dir /tmp/output \\\n  --task_query \'{"status": "complete"}\' \n\n# アノテーションzipを更新してから、アノテーションzipをダウンロードします。\n$ annofabcli statistics visualize --project_id prj1 --output_dir /tmp/output --update_annotation\n\n# WebAPIを実行せずに、作業ディレクトリ（デフォルトは`$XDG_CACHE_HOME/annofabcli`）内のファイルを参照して、統計情報を可視化する。\n$ annofabcli statistics visualize --project_id prj1 --not_update\n\n# prj, prj2 の統計情報を、/tmp/outputにプロジェクトごとに出力します。\n$ annofabcli statistics visualize --project_id prj1 prj2 --output_dir /tmp/output\n\n# prj, prj2, prj3, prj4 の統計情報を、並列処理で出力します。\n$ annofabcli statistics visualize --project_id prj1 prj2 prj3 prj4  --output_dir /tmp/output --parallelism 2\n\n# prj, prj2 の統計情報を、/tmp/outputにプロジェクトごとに出力し、prj1,prj2の統計情報をマージした情報も、`merge`ディレクトリに出力します。\n$ annofabcli statistics visualize --project_id prj1 prj2 --output_dir /tmp/output --merge\n\n```\n\n\n### supplementary list\n補助情報一覧を出力します。\n\n```\n# input_data_idが"id1", "id2"に紐づく補助情報一覧を出力します。\n$ annofabcli supplementary list --project_id prj1 --input_data_id id1 id2\n```\n\n### supplementary put\nCSVに記載された補助情報を登録します。\n\nsupplementary_data_id（省略時は supplementary_data_number）が一致する補助情報が既に存在する場合は、スキップまたは上書きします。\n\n* ヘッダ行なし\n* カンマ区切り\n* 1列目: input_data_id. 必須\n* 2列目: supplementary_data_number. 必須\n* 3列目: supplementary_data_name. 必須\n* 4列目: supplementary_data_path. 必須. 先頭が`file://`の場合、ローカルのファイルを入力データとしてアップロードします。\n* 5列目: supplementary_data_id. 省略可能。省略した場合UUIDv4になる。\n* 6列目: supplementary_data_type. 省略可能. `image` or `text`\n\nCSVのサンプル（`supplementary_data.csv`）です。\n\n```\ninput1,1,data1-1,s3://example.com/data1,id1,\ninput1,2,data1-2,s3://example.com/data2,id2,image\ninput1,3,data1-3,s3://example.com/data3,id3,text\ninput2,1,data2-1,https://example.com/data4,,\ninput2,2,data2-2,file://sample.jpg,,\ninput2,3,data2-3,file:///tmp/sample.jpg,,\n```\n\n\n```\n# supplementary_data.csvに記載されている補助情報を登録する。すでに補助情報が存在する場合はスキップする。\n$ annofabcli supplementary put --project_id prj1 --csv supplementary_data.csv\n\n# supplementary_data.csvに記載されている補助情報を登録する。すでに補助情報が存在する場合は上書きする。\n$ annofabcli supplementary put --project_id prj1 --csv supplementary_data.csv --overwrite\n\n# supplementary_data.csvに記載されている補助情報を、並列処理で登録する（`--yes`オプションが必要）。\n$ annofabcli supplementary put --project_id prj1 --csv supplementary_data.csv --parallelism 2 --yes\n```\n\n\n`supplementary list`コマンドを使えば、プロジェクトに既に登録されている補助情報からCSVを作成できます。\n\n```\n$ annofabcli supplementary list --project_id prj1 --input_data_id id1 id2 \\\n --format csv --output supplementary_data.csv \\\n --csv_format \'{"columns": ["input_data_id", "supplementary_data_number", "supplementary_data_name", "supplementary_data_path", "supplementary_data_id", "supplementary_data_type"], "header":false}\' \n```\n\n\n\n### task cancel_acceptance\n受け入れ完了タスクに対して、受け入れ取り消しにします。\nアノテーションルールを途中で変更したときなどに、利用します。\n\n\n```\n# prj1プロジェクトのタスクを、受け入れ取り消しにする。最後に受入を担当したユーザに割り当てる\n$ annofabcli task cancel_acceptance --project_id prj1 --task_id file://task.txt\n\n# prj1プロジェクトのタスクを、受け入れ取り消しにする。ユーザuser1に割り当てる。\n$ annofabcli task cancel_acceptance --project_id prj1 --task_id file://task.txt --assigned_acceptor_user_id user1\n\n# prj1プロジェクトのタスクを、受け入れ取り消しにする。担当者は未割り当て\n$ annofabcli task cancel_acceptance --project_id prj1 --task_id file://task.txt --not_assign\n\n\n```\n\n\n### task change_operator\nタスクの担当者を変更します。\n\n\n```\n# 指定されたタスクの担当者を \'user1\' に変更する。\n$ annofabcli task change_operator --project_id prj1 --task_id file://task.txt --user_id uer1\n\n# 指定されたタスクの担当者を未割り当てに変更する。\n$ annofabcli task change_operator --project_id prj1 --task_id file://task.txt --not_assign\n\n# 教師付フェーズが未着手のタスクのみ、担当者を変更する\n$ annofabcli task change_operator --project_id prj1 --task_id file://task.txt --not_assign --task_query \'{"status:"not_started", "phase"}\'\n\n```\n\n\n### task complete\nタスクを完了状態にして次のフェーズに進めます。（教師付の提出、検査/受入の合格） 。\n教師付フェーズを完了にする場合は、未回答の検査コメントに対して返信することができます（未回答の検査コメントに対して返信しないと、タスクを提出できないため）。\n検査/受入フェーズを完了する場合は、未処置の検査コメントを対応完了/対応不要状態に変更できます（未処置の検査コメントが残っている状態では、タスクを合格にできないため）。\n\n\n```\n# 教師付フェーズのタスクを提出して、次のフェーズに進めます。未回答の検査コメントがある場合は、スキップします。\n$ annofabcli task complete --project_id prj1 --task_id file://task.txt --phase annotation\n\n# 教師付フェーズのタスクを提出して、次のフェーズに進めます。未回答の検査コメントには「対応しました」と返信します。\n$ annofabcli task complete --project_id prj1 --task_id file://task.txt --phase annotation --reply_comment "対応しました"\n\n# 検査フェーズでステージ2のタスクを合格にして、次のフェーズに進めます。未処置の検査コメントがある場合は次に進めます\n$ annofabcli task complete --project_id prj1 --task_id file://task.txt --phase inspection --phase_stage 2\n\n# 受入フェーズのタスクを合格にして、次のフェーズに進めます。未処置の検査コメントは「対応不要」状態にします。\n$ annofabcli  task complete --project_id prj1 --task_id file://task.txt --phase acceptance \\\n --inspection_status no_correction_required \n```\n\n\n\n### task delete\nタスクを削除します。ただし、作業中/完了状態のタスクは削除できません。デフォルトでは、アノテーションが付与されているタスクは削除できません。\nアノテーションの削除には [annotation delete](#annotation-delete) コマンドを利用できます。\n\n```\n# task_id.txtに記載されたtask_idのタスクを削除します。\n$ annofabcli task delete --project_id prj1 --task_id file://task_id.txt\n\n# アノテーションが付与されているかどうかにかかわらず、タスクを削除します。\n$ annofabcli task delete --project_id prj1 --task_id file://task_id.txt --force\n\n```\n\n\n### task list\nタスク一覧を出力します。\n\n```\n# 受入フェーズで、"usr1"が担当しているタスクの一覧を出力する\n$ annofabcli task list --project_id prj1 --task_query \'{"user_id": "usr1","phase":"acceptance"}\' \n\n# task_id"id1", "id2"のタスクを取得する\n$ annofabcli task list --project_id prj1 --task_id id1 id2\n\n# 休憩中で、過去の担当者が"usr1"であるタスクの一覧を出力する。task.jsonファイルにJSON形式で出力する。\n$ annofabcli task list --project_id prj1 \\\n --task_query \'{"previous_user_id": "usr1","status":"break"}\' --format json --out task.json\n\n# 差し戻されたタスクのtask_idを出力する\n$ annofabcli task list --project_id prj1 --task_query \'{"rejected_only": true}\' --format task_id_list \n\n# タスク情報が記載されたファイルを元にして、タスク一覧を出力します\n# タスク情報が記載されたファイルは、`$ annofabcli project download task`コマンドで取得できます。\n$ annofabcli task list --project_id prj1 --task_json task.json\n\n\n```\n\n#### 出力結果\n\n| project_id                           | task_id                                | phase      | phase_stage | status      | input_data_id_list                       | account_id                           | histories_by_phase                                                                                                                                       | work_time_span | number_of_rejections | started_datetime              | updated_datetime              | sampling | user_id         | username  | worktime_hour       | number_of_rejections_by_inspection | number_of_rejections_by_acceptance |\n|--------------------------------------|----------------------------------------|------------|-------------|-------------|------------------------------------------|--------------------------------------|----------------------------------------------------------------------------------------------------------------------------------------------------------|----------------|----------------------|-------------------------------|-------------------------------|----------|-----------------|-----------|---------------------|------------------------------------|------------------------------------|\n| 12345678-abcd-1234-abcd-1234abcd5678 | 12345678-abcd-1234-abcd-1234abcd5678   | annotation | 1           | break       | [\'12345678-abcd-1234-abcd-1234abcd5678\'] | 12345678-abcd-1234-abcd-1234abcd5678 | [{\'account_id\': \'12345678-abcd-1234-abcd-1234abcd5678\', \'phase\': \'annotation\', \'phase_stage\': 1, \'user_id\': \'user_id1\', \'username\': \'username1\'}] | 539662         | 0                    | 2019-05-08T13:53:21.338+09:00 | 2019-05-08T14:15:07.318+09:00 |          | user_id1 | user_name2 | 0.14990611111111113 | 0                                  | 0                                  |\n\n\n\n\n\n### task list_added_task_history\nタスク履歴情報（フェーズごとの作業時間、担当者、開始日時）を加えたタスク一覧をCSV形式で出力します。\n最初に教師付を開始した日時や担当者を調べるときなどに利用できます。\n\n\n```\n# prj1のタスク全件ファイル、タスク履歴全件ファイルをダウンロードして、タスク一覧のCSVを出力する\n$ annofabcli task list_added_task_history --project_id prj1 --output task.csv\n\n# prj1のタスク全件ファイルの最新版をダウンロードして、タスク一覧のCSVを出力する。タスク履歴全件ファイルはWebAPIの都合上最新化できません。\n$ annofabcli task list_added_task_history --project_id prj1 --output task.csv --latest\n\n# タスク全件ファイルJSON、タスク履歴全件ファイルJSONを参照して、タスク一覧のCSVを出力する\n$ annofabcli task list_added_task_history --project_id prj1 --output task.csv --task_json task.json --task_history_json task_history.json\n```\n\n### task list_task_history\nタスク履歴の一覧を出力します。\n\n\n```\n# prj1の全タスクのタスク履歴の一覧を、CSV形式で出力する\n$ annofabcli task list_task_history --project_id prj1 --output task_history.csv\n\n```\n\n\n### task list_with_json\nタスク全件ファイルから一覧を出力します。\n\n```\n# タスク一覧全件を出力する\n$ annofabcli task list_with_json --project_id prj1 --output task.csv\n\n# タスク全件ファイルを最新化してから、出力する\n$ annofabcli task list_with_json --project_id prj1 --output task.csv --latest\n```\n\n\n\n\n\n### task put\nタスクを作成します。\n\n### CSVファイルに記載された情報を元にタスクを作成する場合\nCSVに記載された情報を元に、タスクを登録します。\nCSVのフォーマットは以下の通りです。\n\n```\n1列目: task_id\n2列目: 空欄（どんな値でもよい）\n3列目: input_data_id\n```\n\nCSVのサンプル（`task.csv`）です。\n\n```\ntask_1,,ca0cb2f9-fec5-49b4-98df-dc34490f9785\ntask_1,,5ac1987e-ca7c-42a0-9c19-b5b23a41836b\ntask_2,,4f2ae4d0-7a38-4f9a-be6f-170ba76aba73\ntask_2,,45ac5852-f20c-4938-9ee9-cc0274401df7\n```\n\n```\n# prj1に、タスク登録処理を投入する。\n$ annofabcli task put --project_id prj1 --csv task.csv\n\n# prj1に、タスク登録処理を投入する。タスク登録が完了するまで待つ.\n$ annofabcli task put --project_id prj1 --csv task.csv --wait\n```\n\n#### 1つのタスクに割り当てる入力データの個数を指定してタスクを作成する場合\n1つのタスクに割り当てる入力データの個数などの情報を、`--by_count`引数に指定します。\nフォーマットは [initiateTasksGeneration](https://annofab.com/docs/api/#operation/initiateTasksGeneration) APIのリクエストボディ `task_generate_rule` を参照してください。\n\n```\n# 「タスクIDのプレフィックスを"sample"、1タスクの入力データ数を10個」でタスクを作成する。\n$ annofabcli task  put --project_id prj1 --by_count \'{"task_id_prefix":"sample","input_data_count":10}\' \n\n# 「タスクIDのプレフィックスを"sample"、1タスクの入力データ数を10個、入力データ名を降順、\n# 既にタスクに使われている入力データも利用」でタスクを作成し、タスク作成が完了するまで待つ\n$ annofabcli task  put --project_id prj1 --by_count \'{"task_id_prefix":"sample","input_data_count":10, \\\n "input_data_order":"random", "allow_duplicate_input_data":true}\' --wait\n```\n\n\n### task reject\nタスクを強制的に差し戻します。差し戻す際に検査コメントを付与することもできます。検査コメントは、画像プロジェクトならばタスク内の先頭の画像の左上(`x=0,y=0`)に、動画プロジェクトなら動画の先頭（`start=0, end=100`)に付与します。\nこの差戻しは差戻しとして扱われず、抜取検査・抜取受入のスキップ判定に影響を及ぼしません。\n\nタスクの状態・フェーズを無視して、フェーズを教師付け(annotation)に、状態を未作業(not started)に変更します。\nタスクの担当者としては、直前の教師付け(annotation)フェーズの担当者を割り当てます。\nこの差戻しは差戻しとして扱われず、抜取検査・抜取受入のスキップ判定に影響を及ぼしません。\n\n```\n# tasks.txtに記載れたタスクを強制的に差し戻す\n# 最後のannotation phaseを担当したユーザを割り当てます（画面と同じ動き）。検査コメントは付与しません。\n$ annofabcli task reject --project_id prj1 --task_id file://tasks.txt \n\n# 受入完了の場合は、受入を取り消してから差し戻します\n$ annofabcli task reject --project_id prj1 --task_id file://tasks.txt --cancel_acceptance\n\n# 「hoge」という検査コメントを付与して、タスクを差し戻します。その際、担当者は割り当てません\n$ annofabcli task reject --project_id prj1 --task_id file://tasks.txt \\\n --comment "hoge" --not_assign\n\n# 差し戻したタスクに、ユーザuser1を割り当てる\n$ annofabcli task reject --project_id prj1 --task_id file://tasks.txt \\\n --comment "hoge" --assigned_annotator_user_id user1\n\n```\n\n\n\n### task update_metadata\nタスクのメタデータを更新します。\nメタデータは常に上書きされることに注意してくだださい。\n\n\n```\n# task1, task2のメタデータを \'{"attr1":"bar"}\' に変更します\n$ annofabcli task update_metadata --project_id prj1 --task_id task1 task2 --metadata \'{"attr1":"bar"}\'\n```\n\n\n\n### task_history list_with_json\nタスク履歴全件ファイルから一覧を出力します。\n\n```\n# タスク履歴一覧全件を出力する\n$ annofabcli task_history list_with_json --project_id prj1 --output task_history.csv\n```\n',
    'author': 'yuji38kwmt',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/kurusugawa-computer/annofab-cli',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)



import argparse
import json
import logging
from pathlib import Path
from typing import List

import requests
from annofabapi.models import AnnotationDataHoldingType

import annofabcli
from annofabcli import AnnofabApiFacade
from annofabcli.common.cli import AbstractCommandLineInterface, ArgumentParser, build_annofabapi_resource_and_login

logger = logging.getLogger(__name__)


class DumpAnnotation(AbstractCommandLineInterface):
    """
    アノテーション情報をダンプする
    """

    def dump_annotation_for_input_data(self, project_id: str, task_id: str, input_data_id: str, task_dir: Path) -> None:
        annotation, _ = self.service.api.get_editor_annotation(project_id, task_id, input_data_id)
        json_path = task_dir / f"{input_data_id}.json"
        json_path.write_text(json.dumps(annotation, ensure_ascii=False), encoding="utf-8")

        details = annotation["details"]
        outer_details = [e for e in details if e["data_holding_type"] == AnnotationDataHoldingType.OUTER.value]
        if len(outer_details) == 0:
            return

        outer_dir = task_dir / input_data_id
        outer_dir.mkdir(exist_ok=True, parents=True)

        for detail in outer_details:
            if not detail["data_holding_type"] == AnnotationDataHoldingType.OUTER.value:
                continue

            outer_file_url = detail["url"]
            response = self.service.api.session.get(outer_file_url)
            if response.status_code != requests.codes.ok:
                logger.warning(
                    f"塗りつぶし画像ファイルのダウンロード失敗しました。"
                    f"status_code={response.status_code}, url={response.url}, text={response.text}"
                )
                continue

            annotation_id = detail["annotation_id"]
            outer_file_path = outer_dir / f"{annotation_id}"
            outer_file_path.write_bytes(response.content)

    def dump_annotation_for_task(self, project_id: str, task_id: str, output_dir: Path) -> bool:
        """
        タスク配下のアノテーションをファイルに保存する。

        Args:
            project_id:
            task_id:
            output_dir: 保存先。配下に"task_id"のディレクトリを作成する。

        Returns:
            アノテーション情報をファイルに保存したかどうか。
        """
        task = self.service.wrapper.get_task_or_none(project_id, task_id)
        if task is None:
            logger.warning(f"task_id = '{task_id}' のタスクは存在しません。スキップします。")
            return False

        input_data_id_list = task["input_data_id_list"]
        task_dir = output_dir / task_id
        task_dir.mkdir(exist_ok=True, parents=True)
        logger.debug(f"task_id = '{task_id}' のアノテーション情報を '{task_dir}' ディレクトリに保存します。")
        for input_data_id in input_data_id_list:
            self.dump_annotation_for_input_data(project_id, task_id, input_data_id, task_dir=task_dir)

        return True

    def dump_annotation(self, project_id: str, task_id_list: List[str], output_dir: Path):
        super().validate_project(project_id, project_member_roles=None)

        project_title = self.facade.get_project_title(project_id)
        logger.info(f"プロジェクト'{project_title}'に対して、タスク{len(task_id_list)} 件のアノテーションをファイルに保存します。")

        output_dir.mkdir(exist_ok=True, parents=True)

        for task_id in task_id_list:
            self.dump_annotation_for_task(project_id, task_id, output_dir=output_dir)

        logger.info(f"処理が完了しました。")

    def main(self):
        args = self.args
        project_id = args.project_id
        task_id_list = annofabcli.common.cli.get_list_from_args(args.task_id)
        output_dir = Path(args.output_dir)
        self.dump_annotation(project_id, task_id_list, output_dir=output_dir)


def main(args):
    service = build_annofabapi_resource_and_login(args)
    facade = AnnofabApiFacade(service)
    DumpAnnotation(service, facade, args).main()


def parse_args(parser: argparse.ArgumentParser):
    argument_parser = ArgumentParser(parser)

    argument_parser.add_project_id()
    argument_parser.add_task_id()

    parser.add_argument("-o", "--output_dir", type=str, required=True, help="出力先ディレクトリのパス")

    parser.set_defaults(subcommand_func=main)


def add_parser(subparsers: argparse._SubParsersAction):
    subcommand_name = "dump"
    subcommand_help = "アノテーション情報をファイルに保存します。"
    description = "指定したタスク配下のアノテーション情報をディレクトリに保存します。アノテーションをバックアップしたいときなどに利用できます。"

    parser = annofabcli.common.cli.add_parser(subparsers, subcommand_name, subcommand_help, description)
    parse_args(parser)



import argparse
import logging

import annofabcli
from annofabcli import AnnofabApiFacade
from annofabcli.common.cli import AbstractCommandLineInterface, ArgumentParser, build_annofabapi_resource_and_login
from annofabcli.common.enums import FormatArgument

logger = logging.getLogger(__name__)


class ListOrganizationMember(AbstractCommandLineInterface):
    """
    組織メンバ一覧を表示する。
    """

    def print_organization_member_list(self, organization_name: str):
        """
        組織メンバ一覧を出力する
        """

        organization_member_list = self.service.wrapper.get_all_organization_members(organization_name)
        logger.debug(f"組織メンバ一覧の件数: {len(organization_member_list)}")
        if len(organization_member_list) == 10000:
            logger.warning("組織メンバ一覧は10,000件で打ち切られている可能性があります。")

        self.print_according_to_format(organization_member_list)

    def main(self):
        args = self.args
        self.print_organization_member_list(args.organization)


def main(args):
    service = build_annofabapi_resource_and_login(args)
    facade = AnnofabApiFacade(service)
    ListOrganizationMember(service, facade, args).main()


def parse_args(parser: argparse.ArgumentParser):
    argument_parser = ArgumentParser(parser)

    parser.add_argument("-org", "--organization", required=True, type=str, help="対象の組織名を指定してください。")

    argument_parser.add_format(
        choices=[FormatArgument.CSV, FormatArgument.JSON, FormatArgument.PRETTY_JSON, FormatArgument.USER_ID_LIST],
        default=FormatArgument.CSV,
    )
    argument_parser.add_output()
    argument_parser.add_csv_format()
    argument_parser.add_query()
    parser.set_defaults(subcommand_func=main)


def add_parser(subparsers: argparse._SubParsersAction):
    subcommand_name = "list"
    subcommand_help = "組織メンバ一覧を出力します。"
    description = "組織メンバ一覧を出力します。"

    parser = annofabcli.common.cli.add_parser(subparsers, subcommand_name, subcommand_help, description)
    parse_args(parser)



"""
複数のプロジェクトに、ユーザを招待する。
"""

import argparse
import logging
from typing import List

import requests
from annofabapi.models import ProjectMemberRole

import annofabcli
import annofabcli.common.cli
from annofabcli import AnnofabApiFacade
from annofabcli.common.cli import AbstractCommandLineInterface, build_annofabapi_resource_and_login

logger = logging.getLogger(__name__)


class InviteUser(AbstractCommandLineInterface):
    """
    ユーザをプロジェクトに招待する
    """

    def assign_role_with_organization(self, organization_name: str, user_id_list: List[str], member_role: str):
        projects = self.service.wrapper.get_all_projects_of_organization(
            organization_name, query_params={"account_id": self.service.api.account_id}
        )

        for project in projects:
            project_id = project["project_id"]
            project_title = project["title"]

            try:
                if not self.facade.my_role_is_owner(project_id):
                    logger.warning(
                        f"オーナではないため、プロジェクトメンバを招待できません。" f"project_id = {project_id}, project_tilte = {project_title}"
                    )
                    continue

                self.service.wrapper.assign_role_to_project_members(project_id, user_id_list, member_role)
                logger.info(f"{project_title}に招待成功. project_id = {project_id}")

            except requests.exceptions.HTTPError as e:
                logger.warning(e)
                logger.warning(f"エラーのため、{project_title} に招待できなかった。")

    def assign_role_with_project_id(self, project_id_list: List[str], user_id_list: List[str], member_role: str):
        for project_id in project_id_list:

            try:
                if not self.facade.my_role_is_owner(project_id):
                    logger.warning(f"オーナではないため、プロジェクトメンバを招待できません。" f"project_id = {project_id}")
                    continue

                project_title = self.service.api.get_project(project_id)[0]["title"]
                self.service.wrapper.assign_role_to_project_members(project_id, user_id_list, member_role)
                logger.info(f"{project_title}に招待成功. project_id={project_id}")

            except requests.exceptions.HTTPError as e:
                logger.warning(e)
                logger.warning(f"エラーのため、招待できなかった。project_id={project_id}")

    def main(self):
        args = self.args

        user_id_list = annofabcli.common.cli.get_list_from_args(args.user_id)

        if args.organization is not None:
            self.assign_role_with_organization(args.organization, user_id_list, args.role)

        elif args.project_id is not None:
            project_id_list = annofabcli.common.cli.get_list_from_args(args.project_id)
            self.assign_role_with_project_id(project_id_list, user_id_list, args.role)


def main(args):
    service = build_annofabapi_resource_and_login(args)
    facade = AnnofabApiFacade(service)
    InviteUser(service, facade, args).main()


def parse_args(parser: argparse.ArgumentParser):
    role_choices = [
        ProjectMemberRole.OWNER.value,
        ProjectMemberRole.WORKER.value,
        ProjectMemberRole.ACCEPTER.value,
        ProjectMemberRole.TRAINING_DATA_USER.value,
    ]

    parser.add_argument(
        "-u",
        "--user_id",
        type=str,
        nargs="+",
        required=True,
        help="招待するユーザのuser_idを指定してください。`file://`を先頭に付けると、一覧が記載されたファイルを指定できます。",
    )

    parser.add_argument("--role", type=str, required=True, choices=role_choices, help="ユーザに割り当てるロール")

    assign_group = parser.add_mutually_exclusive_group(required=True)
    assign_group.add_argument(
        "-p",
        "--project_id",
        type=str,
        nargs="+",
        help="招待するプロジェクトのproject_idを指定してください。`file://`を先頭に付けると、一覧が記載されたファイルを指定できます。",
    )
    assign_group.add_argument("-org", "--organization", type=str, help="組織名を指定すると、組織配下のすべてのプロジェクト（自分が所属している）に招待します。")

    parser.set_defaults(subcommand_func=main)


def add_parser(subparsers: argparse._SubParsersAction):
    subcommand_name = "invite"
    subcommand_help = "複数のプロジェクトに、ユーザを招待する。"
    description = "複数のプロジェクトに、ユーザを招待する。"
    epilog = "オーナロールを持つユーザで実行してください。"

    parser = annofabcli.common.cli.add_parser(subparsers, subcommand_name, subcommand_help, description, epilog=epilog)
    parse_args(parser)



from enum import Enum


class FormatArgument(Enum):
    """
    表示するフォーマット ``--format`` で指定できる値

    Attributes:
        CSV: CSV形式
        JSON: インデントされていないJSON形式
        PRETTY_JSON: インデントされているJSON形式

    """

    #: CSV形式
    CSV = "csv"

    #: 必要最小限の列に絞ったCSV形式
    MINIMAL_CSV = "minimal_csv"

    #: JSON形式
    JSON = "json"

    #: インデントされたJSON形式
    PRETTY_JSON = "pretty_json"

    #: input_data_idの一覧
    INPUT_DATA_ID_LIST = "input_data_id_list"

    #: task_idの一覧
    TASK_ID_LIST = "task_id_list"

    #: inspection_idの一覧
    INSPECTION_ID_LIST = "inspection_id_list"

    #: user_idの一覧
    USER_ID_LIST = "user_id_list"

    #: project_idの一覧
    PROJECT_ID_LIST = "project_id_list"



import argparse
import json
import logging
from pathlib import Path
from typing import Any, Dict, List

from annofabapi.dataclass.annotation import Annotation, AnnotationDetail
from annofabapi.models import AnnotationDataHoldingType, ProjectMemberRole, TaskStatus
from annofabapi.parser import (
    SimpleAnnotationParser,
    SimpleAnnotationParserByTask,
    lazy_parse_simple_annotation_dir_by_task,
)
from annofabapi.utils import can_put_annotation

import annofabcli
from annofabcli import AnnofabApiFacade
from annofabcli.common.cli import AbstractCommandLineInterface, ArgumentParser, build_annofabapi_resource_and_login

logger = logging.getLogger(__name__)


class RestoreAnnotation(AbstractCommandLineInterface):
    """
    アノテーションをリストアする。
    """

    def _to_annotation_detail_for_request(
        self, project_id: str, parser: SimpleAnnotationParser, detail: AnnotationDetail
    ) -> AnnotationDetail:
        """
        Request Bodyに渡すDataClassに変換する。塗りつぶし画像があれば、それをS3にアップロードする。

        Args:
            project_id:
            parser:
            detail: (IN/OUT) １個のアノテーション情報

        Returns:

        """
        if detail.data_holding_type == AnnotationDataHoldingType.OUTER:
            detail.etag = None
            detail.url = None
            data_uri = detail.path

            if data_uri is not None:
                with parser.open_outer_file(data_uri) as f:
                    s3_path = self.service.wrapper.upload_data_to_s3(project_id, f, content_type="image/png")
                    detail.path = s3_path
                    logger.debug(f"{parser.task_id}/{parser.input_data_id}/{data_uri} をS3にアップロードしました。")
            else:
                logger.warning(f"annotattion_id={detail.annotation_id}: data_holding_typeが'outer'なのにpathがNoneです。")

        return detail

    def parser_to_request_body(self, project_id: str, parser: SimpleAnnotationParser) -> Dict[str, Any]:

        annotation: Annotation = Annotation.from_dict(parser.load_json())
        request_details: List[Dict[str, Any]] = []
        for detail in annotation.details:
            request_detail = self._to_annotation_detail_for_request(project_id, parser, detail)

            if request_detail is not None:
                # Enumをシリアライズするため、一度JSONにしてからDictに変換する
                request_details.append(json.loads(request_detail.to_json()))

        request_body = {
            "project_id": project_id,
            "task_id": parser.task_id,
            "input_data_id": parser.input_data_id,
            "details": request_details,
        }

        return request_body

    def put_annotation_for_input_data(self, project_id: str, parser: SimpleAnnotationParser) -> bool:

        task_id = parser.task_id
        input_data_id = parser.input_data_id

        old_annotation, _ = self.service.api.get_editor_annotation(project_id, task_id, input_data_id)

        logger.info(f"task_id={task_id}, input_data_id={input_data_id} : アノテーションをリストアします。")
        request_body = self.parser_to_request_body(project_id, parser)

        updated_datetime = old_annotation["updated_datetime"] if old_annotation is not None else None
        request_body["updated_datetime"] = updated_datetime
        self.service.api.put_annotation(project_id, task_id, input_data_id, request_body=request_body)
        return True

    def put_annotation_for_task(self, project_id: str, task_parser: SimpleAnnotationParserByTask) -> int:

        logger.info(f"タスク'{task_parser.task_id}' のアノテーションをリストアします。")

        success_count = 0
        for parser in task_parser.lazy_parse():
            try:
                if self.put_annotation_for_input_data(project_id, parser):
                    success_count += 1
            except Exception as e:  # pylint: disable=broad-except
                logger.warning(
                    f"task_id={parser.task_id}, input_data_id={parser.input_data_id} のアノテーションのリストアに失敗しました。: {e}"
                )

        logger.info(f"タスク'{task_parser.task_id}'の入力データ {success_count} 個に対してアノテーションをリストアしました。")
        return success_count

    def execute_task(self, project_id: str, task_parser: SimpleAnnotationParserByTask, my_account_id: str) -> bool:
        """
        1個のタスクに対してアノテーションを登録する。

        Args:
            project_id:
            task_parser:
            my_account_id: 自分自身のaccount_id

        Returns:
            1個以上の入力データのアノテーションを変更したか

        """
        task_id = task_parser.task_id
        if not self.confirm_processing(f"task_id={task_id} のアノテーションをリストアしますか？"):
            return False

        logger.info(f"task_id={task_id} に対して処理します。")

        task = self.service.wrapper.get_task_or_none(project_id, task_id)
        if task is None:
            logger.warning(f"task_id = '{task_id}' は存在しません。")
            return False

        if task["status"] in [TaskStatus.WORKING.value, TaskStatus.COMPLETE.value]:
            logger.info(f"タスク'{task_id}'は作業中または受入完了状態のため、アノテーションのリストアをスキップします。 status={task['status']}")
            return False

        if not can_put_annotation(task, my_account_id):
            logger.debug(f"タスク'{task_id}'は、過去に誰かに割り当てられたタスクで、現在の担当者が自分自身でないため、アノテーションのリストアをスキップします。")
            return False

        result_count = self.put_annotation_for_task(project_id, task_parser)
        return result_count > 0

    def main(self):
        args = self.args
        project_id = args.project_id
        annotation_dir_path = Path(args.annotation)

        super().validate_project(project_id, [ProjectMemberRole.OWNER])

        task_id_list = annofabcli.common.cli.get_list_from_args(args.task_id)

        my_account_id = self.service.api.account_id

        # dumpしたアノテーションディレクトリの読み込み
        iter_task_parser = lazy_parse_simple_annotation_dir_by_task(annotation_dir_path)

        success_count = 0
        for task_parser in iter_task_parser:
            try:
                if len(task_id_list) > 0:
                    # コマンドライン引数で --task_idが指定された場合は、対象のタスクのみリストアする
                    if task_parser.task_id in task_id_list:
                        if self.execute_task(project_id, task_parser, my_account_id=my_account_id):
                            success_count += 1
                else:
                    # コマンドライン引数で --task_idが指定されていない場合はすべてをリストアする
                    if self.execute_task(project_id, task_parser, my_account_id=my_account_id):
                        success_count += 1

            except Exception as e:  # pylint: disable=broad-except
                logger.warning(f"task_id={task_parser.task_id} のアノテーションのリストアに失敗しました。: {e}")

        logger.info(f"{success_count} 個のタスクに対してアノテーションをリストアしました。")


def main(args):
    service = build_annofabapi_resource_and_login(args)
    facade = AnnofabApiFacade(service)
    RestoreAnnotation(service, facade, args).main()


def parse_args(parser: argparse.ArgumentParser):
    argument_parser = ArgumentParser(parser)

    argument_parser.add_project_id()

    parser.add_argument(
        "--annotation",
        type=str,
        required=True,
        help="'annotation dump'コマンドの保存先ディレクトリのパスを指定してください。",
    )

    argument_parser.add_task_id(required=False)

    parser.set_defaults(subcommand_func=main)


def add_parser(subparsers: argparse._SubParsersAction):
    subcommand_name = "restore"
    subcommand_help = "'annotation dump'コマンドで保存したファイルから、アノテーション情報をリストアします。"
    description = (
        "'annotation dump'コマンドで保存したファイルから、アノテーション情報をリストアします。"
        "ただし、作業中/完了状態のタスク、または「過去に割り当てられていて現在の担当者が自分自身でない」タスクはリストアできません。"
    )
    epilog = "チェッカーまたはオーナロールを持つユーザで実行してください。"

    parser = annofabcli.common.cli.add_parser(subparsers, subcommand_name, subcommand_help, description, epilog=epilog)
    parse_args(parser)



import enum
from typing import Any, Dict, List, Optional

import annofabapi
import more_itertools
from annofabapi.models import (
    AnnotationSpecsHistory,
    InputData,
    Inspection,
    OrganizationMember,
    ProjectMember,
    Task,
    TaskHistory,
    TaskPhase,
)
from annofabapi.utils import get_number_of_rejections

from annofabcli.common.utils import isoduration_to_hour


class MessageLocale(enum.Enum):
    EN = "en-US"
    JA = "ja-JP"


class AddProps:
    """
    WebAPIから取得したコンテンツに、属性を付与する。
    """

    #: 組織メンバ一覧のキャッシュ
    _organization_members: Optional[List[OrganizationMember]] = None
    _project_member_list: Optional[List[ProjectMember]] = None

    def __init__(self, service: annofabapi.Resource, project_id: str):
        self.service = service
        self.project_id = project_id
        self.organization_name = self._get_organization_name_from_project_id(project_id)

        annotation_specs, _ = self.service.api.get_annotation_specs(project_id)
        self.specs_labels: List[Dict[str, Any]] = annotation_specs["labels"]
        self.specs_inspection_phrases: List[Dict[str, Any]] = annotation_specs["inspection_phrases"]

    @staticmethod
    def millisecond_to_hour(millisecond: int):
        return millisecond / 1000 / 3600

    @staticmethod
    def get_message(i18n_messages: Dict[str, Any], locale: MessageLocale) -> Optional[str]:
        messages: List[Dict[str, Any]] = i18n_messages["messages"]
        dict_message = more_itertools.first_true(messages, pred=lambda e: e["lang"] == locale.value)
        if dict_message is not None:
            return dict_message["message"]
        else:
            return None

    @staticmethod
    def add_properties_of_project(target: Dict[str, Any], project_title: str) -> Dict[str, Any]:
        target["project_title"] = project_title
        return target

    def _add_user_info(self, target: Any):
        user_id = None
        username = None

        account_id = target["account_id"]
        if account_id is not None:
            member = self.get_project_member_from_account_id(account_id)
            if member is not None:
                user_id = member["user_id"]
                username = member["username"]

        target["user_id"] = user_id
        target["username"] = username
        return target

    def get_project_member_from_account_id(self, account_id: str) -> Optional[ProjectMember]:
        if self._project_member_list is None:
            project_member_list = self.service.wrapper.get_all_project_members(
                self.project_id, query_params={"include_inactive_member": True}
            )
            self._project_member_list = project_member_list
        else:
            project_member_list = self._project_member_list

        return more_itertools.first_true(project_member_list, pred=lambda e: e["account_id"] == account_id)

    def get_organization_member_from_account_id(self, account_id: str) -> Optional[OrganizationMember]:
        """
        account_idから組織メンバを取得する.
        内部で保持している組織メンバ一覧を参照する。

        Args:
            project_id:
            accoaunt_id:

        Returns:
            組織メンバ
        """

        def update_organization_members():
            self._organization_members = self.service.wrapper.get_all_organization_members(self.organization_name)

        def get_member():
            member = more_itertools.first_true(self._organization_members, pred=lambda e: e["account_id"] == account_id)
            return member

        if self._organization_members is not None:
            member = get_member()
            if member is not None:
                return member

            else:
                update_organization_members()
                return get_member()

        else:
            update_organization_members()
            return get_member()

    def _get_organization_name_from_project_id(self, project_id: str) -> str:
        """
        project_Idから組織名を取得する。
        """
        organization, _ = self.service.api.get_organization_of_project(project_id)
        return organization["organization_name"]

    def get_phrase_name(self, phrase_id, locale: MessageLocale) -> Optional[str]:
        phrase: Optional[Dict[str, Any]] = more_itertools.first_true(
            self.specs_inspection_phrases, pred=lambda e: e["id"] == phrase_id
        )
        if phrase is None:
            return None

        return self.get_message(phrase["text"], locale)

    def get_label_name(self, label_id: str, locale: MessageLocale) -> Optional[str]:
        label = more_itertools.first_true(self.specs_labels, pred=lambda e: e["label_id"] == label_id)
        if label is None:
            return None

        return self.get_message(label["label_name"], locale)

    def get_additional_data_name(
        self, additional_data_definition_id: str, locale: MessageLocale, label_id: Optional[str] = None
    ) -> Optional[str]:
        def _get_additional_data_name(arg_additional_data_definitions: List[Dict[str, Any]]) -> Optional[str]:
            additional_data = more_itertools.first_true(
                arg_additional_data_definitions,
                pred=lambda e: e["additional_data_definition_id"] == additional_data_definition_id,
            )
            if additional_data is None:
                return None
            return self.get_message(additional_data["name"], locale)

        if label_id is not None:
            label = more_itertools.first_true(self.specs_labels, pred=lambda e: e["label_id"] == label_id)
            if label is None:
                return None
            else:
                return _get_additional_data_name(label["additional_data_definitions"])
        else:
            for label in self.specs_labels:
                additional_data_name = _get_additional_data_name(label["additional_data_definitions"])
                if additional_data_name is not None:
                    return additional_data_name

            return None

    def add_properties_to_annotation_specs_history(
        self, annotation_specs_history: AnnotationSpecsHistory
    ) -> AnnotationSpecsHistory:
        """
        アノテーション仕様の履歴に、以下のキーを追加する.
        user_id
        username

        Args:
            annotation_specs_history:

        Returns:
            annotation_specs_history
        """
        return self._add_user_info(annotation_specs_history)

    def add_properties_to_inspection(
        self, inspection: Inspection, detail: Optional[Dict[str, Any]] = None
    ) -> Inspection:
        """
        検査コメントに、以下のキーを追加する.
        commenter_user_id
        commenter_username
        phrase_names_en
        phrase_names_ja
        label_name_en
        label_name_en

        Args:
            inspection:
            detail: 検査コメント情報に追加する詳細な情報

        Returns:

        """

        def add_commenter_info():
            commenter_user_id = None
            commenter_username = None

            commenter_account_id = inspection["commenter_account_id"]
            if commenter_account_id is not None:
                member = self.get_project_member_from_account_id(commenter_account_id)
                if member is not None:
                    commenter_user_id = member["user_id"]
                    commenter_username = member["username"]

            inspection["commenter_user_id"] = commenter_user_id
            inspection["commenter_username"] = commenter_username

        add_commenter_info()
        inspection["phrase_names_en"] = [self.get_phrase_name(e, MessageLocale.EN) for e in inspection["phrases"]]
        inspection["phrase_names_ja"] = [self.get_phrase_name(e, MessageLocale.JA) for e in inspection["phrases"]]

        inspection["label_name_en"] = self.get_label_name(inspection["label_id"], MessageLocale.EN)
        inspection["label_name_ja"] = self.get_label_name(inspection["label_id"], MessageLocale.JA)

        if detail is not None:
            inspection.update(detail)

        return inspection

    def add_properties_to_task(self, task: Task) -> Task:
        """
        タスク情報に、以下のキーを追加する.

        * user_id
        * username
        * worktime_hour
        * number_of_rejections_by_inspection
        * number_of_rejections_by_acceptance

        Args:
            task:

        Returns:
            Task情報

        """

        self._add_user_info(task)
        task["worktime_hour"] = self.millisecond_to_hour(task["work_time_span"])

        histories = [self._add_user_info(e) for e in task["histories_by_phase"]]
        task["histories_by_phase"] = histories

        task["number_of_rejections_by_inspection"] = get_number_of_rejections(histories, TaskPhase.INSPECTION)
        task["number_of_rejections_by_acceptance"] = get_number_of_rejections(histories, TaskPhase.ACCEPTANCE)

        # number_of_rejectionsは非推奨なプロパティで、number_of_rejections_by_inspection/number_of_rejections_by_acceptanceと矛盾する場合があるので、削除する
        task.pop("number_of_rejections", None)
        return task

    def add_properties_to_task_history(self, task_history: TaskHistory) -> TaskHistory:
        """
        タスク履歴情報に、以下のキーを追加する.

        * user_id
        * username
        * worktime_hour

        Args:
            task:

        Returns:
            Task情報

        """

        self._add_user_info(task_history)
        task_history["worktime_hour"] = isoduration_to_hour(task_history["accumulated_labor_time_milliseconds"])
        return task_history

    @staticmethod
    def add_properties_to_input_data(input_data: InputData, task_id_list: List[str]) -> InputData:
        """
        入力データ情報に、以下のキーを追加する。

        * parent_task_id_list

        Args:
            input_data:
            task_id_list:

        Returns:
            入力データ情報

        """
        input_data["parent_task_id_list"] = task_id_list
        return input_data



import argparse
import logging
from typing import Any, Dict, List

import pandas

import annofabcli
import annofabcli.common.cli
from annofabcli import AnnofabApiFacade
from annofabcli.common.cli import AbstractCommandLineInterface, ArgumentParser, build_annofabapi_resource_and_login

logger = logging.getLogger(__name__)


class TaskProgress(AbstractCommandLineInterface):
    """
    タスクの進捗状況を出力する。
    """

    def get_task_statistics(self, project_id: str) -> List[Dict[str, Any]]:
        """
        タスクの進捗状況をCSVに出力するための dict 配列を作成する。

        Args:
            project_id:

        Returns:
            タスクの進捗状況に対応するdict配列

        """
        task_statistics = self.service.wrapper.get_task_statistics(project_id)
        row_list: List[Dict[str, Any]] = []
        for stat_by_date in task_statistics:
            date = stat_by_date["date"]
            task_stat_list = stat_by_date["tasks"]
            for task_stat in task_stat_list:
                task_stat["date"] = date
            row_list.extend(task_stat_list)
        return row_list

    def list_task_progress(self, project_id: str) -> None:
        super().validate_project(project_id, project_member_roles=None)

        task_stat_list = self.get_task_statistics(project_id)
        if len(task_stat_list) == 0:
            logger.info(f"タスクの進捗状況の情報がないため、出力しません。")
            return

        df = pandas.DataFrame(task_stat_list)
        # 出力対象の列を指定する
        target_df = df[["date", "phase", "status", "count"]]

        annofabcli.utils.print_csv(target_df, output=self.output, to_csv_kwargs=self.csv_format)

    def main(self):
        args = self.args

        project_id = args.project_id
        self.list_task_progress(project_id)


def parse_args(parser: argparse.ArgumentParser):
    argument_parser = ArgumentParser(parser)

    argument_parser.add_project_id()
    argument_parser.add_csv_format()
    argument_parser.add_output()

    parser.set_defaults(subcommand_func=main)


def main(args):
    service = build_annofabapi_resource_and_login(args)
    facade = AnnofabApiFacade(service)
    TaskProgress(service, facade, args).main()


def add_parser(subparsers: argparse._SubParsersAction):
    subcommand_name = "list_task_progress"
    subcommand_help = "タスク進捗情報を出力する"
    description = "タスク進捗状況をCSV形式で出力する。"
    parser = annofabcli.common.cli.add_parser(subparsers, subcommand_name, subcommand_help, description=description)
    parse_args(parser)



"""
Command Line Interfaceの共通部分
"""

import abc
import argparse
import dataclasses
import getpass
import json
import logging
import os
from typing import Any, Dict, List, Optional, Tuple

import annofabapi
import jmespath
import pandas
import requests
from annofabapi.api import DEFAULT_ENDPOINT_URL
from annofabapi.exceptions import AnnofabApiException
from annofabapi.models import OrganizationMemberRole, ProjectMemberRole

import annofabcli
from annofabcli.common.dataclasses import WaitOptions
from annofabcli.common.enums import FormatArgument
from annofabcli.common.facade import AnnofabApiFacade
from annofabcli.common.typing import InputDataSize

logger = logging.getLogger(__name__)

DEFAULT_CSV_FORMAT = {"encoding": "utf_8_sig", "index": False}


def build_annofabapi_resource_and_login(args: argparse.Namespace) -> annofabapi.Resource:
    """
    annofabapi.Resourceインスタンスを生成したあと、ログインする。

    Args:
        args: コマンドライン引数の情報

    Returns:
        annofabapi.Resourceインスタンス

    """

    service = build_annofabapi_resource(args)

    try:
        service.api.login()
        return service

    except requests.exceptions.HTTPError as e:
        if e.response.status_code == requests.codes.unauthorized:
            raise annofabcli.exceptions.AuthenticationError(service.api.login_user_id)
        raise e


def add_parser(
    subparsers: argparse._SubParsersAction,
    command_name: str,
    command_help: str,
    description: str,
    is_subcommand: bool = True,
    epilog: Optional[str] = None,
) -> argparse.ArgumentParser:
    """
    サブコマンド用にparserを追加する

    Args:
        subparsers:
        command_name:
        command_help: 1階層上のコマンドヘルプに表示される コマンドの説明（簡易的な説明）
        description: ヘルプ出力に表示される説明（詳細な説明）
        is_subcommand: サブコマンドかどうか. `annofabcli project`はコマンド、`annofabcli project list`はサブコマンドとみなす。
        epilog: ヘルプ出力後に表示される内容。デフォルトはNoneです。

    Returns:
        サブコマンドのparser

    """
    parents = [create_parent_parser()] if is_subcommand else []
    parser = subparsers.add_parser(
        command_name,
        parents=parents,
        description=description,
        help=command_help,
        epilog=epilog,
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    parser.set_defaults(command_help=parser.print_help)
    return parser


def create_parent_parser() -> argparse.ArgumentParser:
    """
    共通の引数セットを生成する。
    """
    parent_parser = argparse.ArgumentParser(add_help=False)
    group = parent_parser.add_argument_group("global optional arguments")

    group.add_argument("--yes", action="store_true", help="処理中に現れる問い合わせに対して、常に'yes'と回答します。")

    # EXAMPLE_CREDENTAILS = '{"user_id": "test_user", "password": "test_password"}'
    # group.add_argument(
    #     "--credentials",
    #     type=str,
    #     help=f"AnnoFabにログインするユーザの認証情報をJSON形式で指定します。"
    #     f"(ex) `{EXAMPLE_CREDENTAILS}` ."
    #     f"`file://`を先頭に付けると、JSON形式のファイルを指定できます。",
    # )

    group.add_argument(
        "--endpoint_url", type=str, help=f"AnnoFab WebAPIのエンドポイントを指定します。指定しない場合は'{DEFAULT_ENDPOINT_URL}'です。"
    )

    group.add_argument(
        "--logdir", type=str, default=".log", help="ログファイルを保存するディレクトリを指定します。指定しない場合は`.log`ディレクトリ'にログファイルが保存されます。"
    )

    group.add_argument("--disable_log", action="store_true", help="ログを無効にします。")

    group.add_argument(
        "--logging_yaml",
        type=str,
        help="ロギグングの設定ファイル(YAML)を指定します。指定した場合、`--logdir`オプションは無視されます。"
        "指定しない場合、デフォルトのロギングが設定されます。"
        "設定ファイルの書き方は https://docs.python.org/ja/3/howto/logging.html 参照してください。",
    )
    return parent_parser


def get_list_from_args(str_list: Optional[List[str]] = None) -> List[str]:
    """
    文字列のListのサイズが1で、プレフィックスが`file://`ならば、ファイルパスとしてファイルを読み込み、行をListとして返す。
    そうでなければ、引数の値をそのまま返す。
    ただしNoneの場合は空Listを返す。

    Args:
        str_list: コマンドライン引数で指定されたリスト、またはfileスキームのURL

    Returns:
        コマンドライン引数で指定されたリスト。
    """
    if str_list is None or len(str_list) == 0:
        return []

    if len(str_list) > 1:
        return str_list

    str_value = str_list[0]
    path = annofabcli.utils.get_file_scheme_path(str_value)
    if path is not None:
        return annofabcli.utils.read_lines_except_blank_line(path)
    else:
        return str_list


def get_csv_format_from_args(target: Optional[str] = None) -> Dict[str, Any]:
    """
    コマンドライン引数の値から csv_format を取得する。
    Default: {"encoding": "utf_8_sig", "index": False}

    """
    csv_format = DEFAULT_CSV_FORMAT.copy()
    if target is not None:
        arg_csv_format = get_json_from_args(target)
        csv_format.update(arg_csv_format)

    return csv_format


def get_json_from_args(target: Optional[str] = None) -> Any:
    """
    JSON形式をPythonオブジェクトに変換する。
    プレフィックスが`file://`ならば、ファイルパスとしてファイルを読み込み、Pythonオブジェクトを返す。
    """

    if target is None:
        return None

    path = annofabcli.utils.get_file_scheme_path(target)
    if path is not None:
        with open(path, encoding="utf-8") as f:
            return json.load(f)
    else:
        return json.loads(target)


def get_input_data_size(str_input_data_size: str) -> Optional[InputDataSize]:
    """400x300を(400,300)に変換する"""
    splited_list = str_input_data_size.split("x")
    if len(splited_list) < 2:
        return None

    return (int(splited_list[0]), int(splited_list[1]))


def get_wait_options_from_args(
    dict_wait_options: Optional[Dict[str, Any]], default_wait_options: WaitOptions
) -> WaitOptions:
    """
    デフォルト値とマージして、wait_optionsを取得する。

    Args:
        dict_wait_options: dictのwait_options(コマンドラインから取得した値など）
        default_wait_options: デフォルトのwait_options

    Returns:
        デフォルト値とマージしたwait_options

    """
    if dict_wait_options is not None:
        dataclasses.asdict(default_wait_options)
        return WaitOptions.from_dict({**dataclasses.asdict(default_wait_options), **dict_wait_options})
    else:
        return default_wait_options


def load_logging_config_from_args(args: argparse.Namespace) -> None:
    """
    args情報から、logging設定ファイルを読み込む.
    以下のコマンドライン引数からlogging設定ファイルを読み込む。
    ``--disable_log`` が指定されている場合は、loggerを設定しない。

    * --logdir
    * --disable_log
    * --logging_yaml

    Args:
        args: Command引数情報
    """

    if args.disable_log:
        return

    annofabcli.utils.load_logging_config(args.logdir, args.logging_yaml)


def get_endpoint_url(args: argparse.Namespace) -> str:
    """
    AnnoFab WebAPIのエンドポイントURLを、以下の優先順位で取得する。

    1. コマンドライン引数 ``--endpoint_url``
    2. 環境変数 ``ANNOFAB_ENDPOINT_URL``

    取得できない場合は、デフォルトの ``https://annofab.com`` を返す。

    Args:
        args: コマンドライン引数情報

    Returns:
        AnnoFab WebAPIのエンドポイントURL

    """
    endpoint_url = args.endpoint_url
    if endpoint_url is not None:
        return endpoint_url

    endpoint_url = os.environ.get("ANNOFAB_ENDPOINT_URL")
    if endpoint_url is not None:
        return endpoint_url

    return DEFAULT_ENDPOINT_URL


def build_annofabapi_resource(args: argparse.Namespace) -> annofabapi.Resource:
    """
    annofabapi.Resourceインスタンスを生成する。
    以下の順にAnnoFabの認証情報を読み込む。
    1. `.netrc`ファイル
    2. 環境変数`ANNOFAB_USER_ID` , `ANNOFAB_PASSWORD`

    認証情報を読み込めなかった場合は、標準入力からUser IDとパスワードを入力させる。

    Returns:
        annofabapi.Resourceインスタンス

    """
    endpoint_url = get_endpoint_url(args)
    if endpoint_url != DEFAULT_ENDPOINT_URL:
        logger.info(f"AnnoFab WebAPIのエンドポイントURL: {endpoint_url}")

    # # コマンドライン引数から認証情報を取得する
    # dict_credentials = annofabcli.common.cli.get_json_from_args(args.credentials)
    # if dict_credentials is not None:
    #     return annofabapi.build(
    #         dict_credentials.get("user_id"), dict_credentials.get("password"), endpoint_url=endpoint_url
    #     )

    # '.netrc'ファイルから認証情報を取得する
    try:
        return annofabapi.build_from_netrc(endpoint_url)
    except AnnofabApiException:
        logger.debug("`.netrc`ファイルにはAnnoFab認証情報が存在しなかった")

    # 環境変数から認証情報を取得する
    try:
        return annofabapi.build_from_env(endpoint_url)
    except AnnofabApiException:
        logger.debug("`環境変数`ANNOFAB_USER_ID` or  `ANNOFAB_PASSWORD`が空だった")

    # 標準入力から入力させる
    login_user_id = ""
    while login_user_id == "":
        login_user_id = input("Enter AnnoFab User ID: ")

    login_password = ""
    while login_password == "":
        login_password = getpass.getpass("Enter AnnoFab Password: ")

    return annofabapi.build(login_user_id, login_password, endpoint_url=endpoint_url)


def prompt_yesno(msg: str) -> bool:
    """
    標準入力で yes, noを選択できるようにする。
    Args:
        msg: 確認メッセージ

    Returns:
        True: Yes, False: No

    """
    while True:
        choice = input(f"{msg} [y/N] : ")
        if choice == "y":
            return True

        elif choice == "N":
            return False


def prompt_yesnoall(msg: str) -> Tuple[bool, bool]:
    """
    標準入力で yes, no, all(すべてyes)を選択できるようにする。
    Args:
        msg: 確認メッセージ

    Returns:
        Tuple[yesno, allflag]. yesno:Trueならyes. allflag: Trueならall.

    """
    while True:
        choice = input(f"{msg} [y/N/ALL] : ")
        if choice == "y":
            return True, False

        elif choice == "N":
            return False, False

        elif choice == "ALL":
            return True, True


class ArgumentParser:
    """
    共通のコマンドライン引数を追加するためのクラス
    """

    def __init__(self, parser: argparse.ArgumentParser):
        self.parser = parser

    def add_project_id(self, help_message: Optional[str] = None):
        """
        '--project_id` 引数を追加
        """
        if help_message is None:
            help_message = "対象のプロジェクトのproject_idを指定します。"

        self.parser.add_argument("-p", "--project_id", type=str, required=True, help=help_message)

    def add_task_id(self, required: bool = True, help_message: Optional[str] = None):
        """
        '--task_id` 引数を追加
        """
        if help_message is None:
            help_message = "対象のタスクのtask_idを指定します。" + "`file://`を先頭に付けると、task_idの一覧が記載されたファイルを指定できます。"

        self.parser.add_argument("-t", "--task_id", type=str, required=required, nargs="+", help=help_message)

    def add_input_data_id(self, required: bool = True, help_message: Optional[str] = None):
        """
        '--input_data_id` 引数を追加
        """
        if help_message is None:
            help_message = "対象の入力データのinput_data_idを指定します。" + "`file://`を先頭に付けると、input_data_idの一覧が記載されたファイルを指定できます。"

        self.parser.add_argument("-i", "--input_data_id", type=str, required=required, nargs="+", help=help_message)

    def add_format(self, choices: List[FormatArgument], default: FormatArgument, help_message: Optional[str] = None):
        """
        '--format` 引数を追加
        """
        if help_message is None:
            help_message = f"出力フォーマットを指定します。指定しない場合は、{default.value} フォーマットになります。"

        self.parser.add_argument(
            "-f", "--format", type=str, choices=[e.value for e in choices], default=default.value, help=help_message
        )

    def add_csv_format(self, help_message: Optional[str] = None):
        """
        '--csv_format` 引数を追加
        """
        if help_message is None:
            help_message = (
                "CSVのフォーマットをJSON形式で指定します。`--format`が`csv`でないときは、このオプションは無視されます。"
                "`file://`を先頭に付けると、JSON形式のファイルを指定できます。"
                "指定した値は、[pandas.DataFrame.to_csv](https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.DataFrame.to_csv.html) の引数として渡されます。"  # noqa: E501
            )

        self.parser.add_argument("--csv_format", type=str, help=help_message)

    def add_output(self, required: bool = False, help_message: Optional[str] = None):
        """
        '--output` 引数を追加
        """
        if help_message is None:
            help_message = "出力先のファイルパスを指定します。指定しない場合は、標準出力に出力されます。"

        self.parser.add_argument("-o", "--output", type=str, required=required, help=help_message)

    def add_query(self, help_message: Optional[str] = None):
        """
        '--query` 引数を追加
        """
        if help_message is None:
            help_message = "JMESPath形式で指定します。出力結果の抽出や、出力内容の変更に利用できます。"

        self.parser.add_argument("-q", "--query", type=str, help=help_message)

    def add_task_query(self, required: bool = False, help_message: Optional[str] = None):
        if help_message is None:
            help_message = (
                "タスクを絞り込むためのクエリ条件をJSON形式で指定します。指定しない場合はすべてのタスクを取得します。"
                "`file://`を先頭に付けると、JSON形式のファイルを指定できます。"
                "使用できるキーは、phase, phase_stage, status, user_id, account_id, no_user (bool値)  のみです。"
            )
        self.parser.add_argument("-tq", "--task_query", type=str, required=required, help=help_message)


class AbstracCommandCinfirmInterface(abc.ABC):
    """
    コマンドライン上でpromptを表示するときのインターフェイズ
    """

    def __init__(self, all_yes: bool = False):
        self.all_yes = all_yes

    def confirm_processing(self, confirm_message: str) -> bool:
        """
        `all_yes`属性を見て、処理するかどうかユーザに問い合わせる。
        "ALL"が入力されたら、`all_yes`属性をTrueにする

        Returns:
            True: Yes, False: No

        """
        if self.all_yes:
            return True
        yes, all_yes = prompt_yesnoall(confirm_message)
        if all_yes:
            self.all_yes = True
        return yes


class AbstractCommandLineInterface(abc.ABC):
    """
    CLI用の抽象クラス
    """

    #: annofabapi.Resourceインスタンス
    service: annofabapi.Resource

    #: AnnofabApiFacadeインスタンス
    facade: AnnofabApiFacade

    #: Trueならば、処理中に現れる問い合わせに対して、常に'yes'と回答したものとして処理する。
    all_yes: bool = False

    #: JMesPath
    query: Optional[str] = None

    #: 出力先
    output: Optional[str] = None

    #: CSVのフォーマット
    csv_format: Optional[Dict[str, Any]] = None

    #: 出力フォーマット
    str_format: Optional[str] = None

    def __init__(self, service: annofabapi.Resource, facade: AnnofabApiFacade, args: argparse.Namespace):
        self.service = service
        self.facade = facade
        self.args = args
        self.process_common_args(args)

    def process_common_args(self, args: argparse.Namespace):
        """
        共通のコマンドライン引数を処理する。
        Args:
            args: コマンドライン引数
        """
        self.all_yes = args.yes
        if hasattr(args, "query"):
            self.query = args.query

        if hasattr(args, "csv_format"):
            self.csv_format = annofabcli.common.cli.get_csv_format_from_args(args.csv_format)

        if hasattr(args, "output"):
            self.output = args.output

        if hasattr(args, "format"):
            self.str_format = args.format

        logger.info(f"args: {args}")

    def validate_project(
        self,
        project_id,
        project_member_roles: Optional[List[ProjectMemberRole]] = None,
        organization_member_roles: Optional[List[OrganizationMemberRole]] = None,
    ):
        """
        プロジェクト or 組織に対して、必要な権限が付与されているかを確認する。

        Args:
            project_id:
            project_member_roles: プロジェクトメンバロールの一覧. Noneの場合はチェックしない。
            organization_member_roles: 組織メンバロールの一覧。Noneの場合はチェックしない。

        Raises:
             AuthorizationError: 自分自身のRoleがいずれかのRoleにも合致しなければ、AuthorizationErrorが発生する。

        """
        self.facade.validate_project(
            project_id=project_id,
            project_member_roles=project_member_roles,
            organization_member_roles=organization_member_roles,
        )

    def confirm_processing(self, confirm_message: str) -> bool:
        """
        `all_yes`属性を見て、処理するかどうかユーザに問い合わせる。
        "ALL"が入力されたら、`all_yes`属性をTrueにする

        Args:
            task_id: 処理するtask_id
            confirm_message: 確認メッセージ

        Returns:
            True: Yes, False: No

        """
        if self.all_yes:
            return True

        yes, all_yes = prompt_yesnoall(confirm_message)

        if all_yes:
            self.all_yes = True

        return yes

    def confirm_processing_task(self, task_id: str, confirm_message: str) -> bool:
        """
        タスクに対して処理するかどうか問い合わせる。
        `all_yes`属性も設定する。

        Args:
            task_id: 処理するtask_id
            confirm_message: 確認メッセージ

        Returns:
            Trueならば対象のタスクを処理する。

        """
        if self.all_yes:
            return True

        yes, all_yes = prompt_yesnoall(confirm_message)

        if not yes:
            logger.info(f"task_id = {task_id} をスキップします。")
            return False

        if all_yes:
            self.all_yes = True

        return True

    def search_with_jmespath_expression(self, target: Any):
        """
        インスタンスで保持しているJMespath情報で、targetの中身を探す。
        Args:
            target: 検索対象

        Returns:
            JMesPathで検索した結果。``self.query`` がNoneなら引数 ``target`` を返す。

        """
        if self.query is not None:
            return jmespath.search(self.query, target)
        return target

    def print_csv(self, df: pandas.DataFrame):
        annofabcli.utils.print_csv(df, output=self.output, to_csv_kwargs=self.csv_format)

    def print_according_to_format(self, target: Any):
        target = self.search_with_jmespath_expression(target)

        annofabcli.utils.print_according_to_format(
            target, arg_format=FormatArgument(self.str_format), output=self.output, csv_format=self.csv_format
        )



"""
プロジェクト間の差分を表示する。
"""

import argparse
import copy
import functools
import logging
import pprint
from enum import Enum
from typing import Any, Dict, List, Tuple

import annofabapi
import dictdiffer
import more_itertools
from annofabapi.models import ProjectMemberRole

import annofabcli
import annofabcli.common.cli
from annofabcli import AnnofabApiFacade
from annofabcli.common.cli import AbstractCommandLineInterface, build_annofabapi_resource_and_login

DiffResult = Tuple[bool, str]
"""差分があるかどうかと、差分メッセージ"""

logger = logging.getLogger(__name__)


class DiffTarget(Enum):
    """
    比較する項目
    """

    ANNOTATION_LABELS = "annotation_labels"
    INSPECTION_PHRASES = "inspection_phrases"
    MEMBERS = "members"
    SETTINGS = "settings"


def sorted_inspection_phrases(phrases: List[Dict[str, Any]]):
    return sorted(phrases, key=lambda e: e["id"])


def sorted_project_members(project_members: List[Dict[str, Any]]):
    return sorted(project_members, key=lambda e: e["user_id"])


def create_ignored_label(label: Dict[str, Any]):
    """
    比較対象外のkeyを削除したラベル情報を生成する
    """

    copied_label = copy.deepcopy(label)
    copied_label.pop("label_id", None)

    additional_data_definitions = copied_label["additional_data_definitions"]
    for additional_data in additional_data_definitions:
        additional_data.pop("additional_data_definition_id", None)
        choices = additional_data["choices"]
        for choice in choices:
            choice.pop("choice_id", None)

    return copied_label


class DiffProjecs(AbstractCommandLineInterface):
    """
    プロジェクト間の差分を表示する
    """

    def __init__(self, service: annofabapi.Resource, facade: AnnofabApiFacade, args: argparse.Namespace):
        super().__init__(service, facade, args)

        project_id1 = args.project_id1
        project_id2 = args.project_id2
        project_title1 = self.facade.get_project_title(project_id1)
        project_title2 = self.facade.get_project_title(project_id2)

        self.project_title1 = project_title1
        self.project_title2 = project_title2

    def diff_project_members(self, project_id1: str, project_id2: str) -> DiffResult:
        """
        プロジェクト間のプロジェクトメンバの差分を表示する。
        Args:
            project_id1: 比較対象のプロジェクトのproject_id
            project_id2: 比較対象のプロジェクトのproject_id

        Returns:
            Tuple(差分があるかどうか, 差分メッセージ)

        """
        logger.info("=== プロジェクトメンバの差分 ===")

        diff_message = ""

        project_members1 = self.service.wrapper.get_all_project_members(project_id1)
        project_members2 = self.service.wrapper.get_all_project_members(project_id2)

        # プロジェクトメンバは順番に意味がないので、ソートしたリストを比較する
        sorted_members1 = sorted_project_members(project_members1)
        sorted_members2 = sorted_project_members(project_members2)

        user_ids1 = [e["user_id"] for e in sorted_members1]
        user_ids2 = [e["user_id"] for e in sorted_members2]

        if user_ids1 != user_ids2:
            diff_message += (
                f"### user_idのListに差分あり\n"
                f"set(user_ids1) - set(user_ids2) = {set(user_ids1) - set(user_ids2)}\n"
                f"set(user_ids2) - set(user_ids1) = {set(user_ids2) - set(user_ids1)}\n"
            )

            return True, diff_message

        is_different = False
        for member1, member2 in zip(sorted_members1, sorted_members2):
            ignored_key = {"updated_datetime", "created_datetime", "project_id"}
            diff_result = list(dictdiffer.diff(member1, member2, ignore=ignored_key))
            if len(diff_result) > 0:
                is_different = True
                diff_message += f"差分のあるuser_id: {member1['user_id']}\n" f"{pprint.pformat(diff_result)}\n"

        if not is_different:
            logger.info("プロジェクトメンバは同じ")

        return is_different, diff_message

    def validate_duplicated(self, label_names1: List[str], label_names2: List[str]) -> DiffResult:
        """
        label_nameが重複しているか確認する
        Args:
            label_names1:
            label_names2:

        Returns:
            Tuple(差分があるかどうか, 差分メッセージ)
            Trueなら、label_names1 or label_names2が重複している

        """

        diff_message = ""

        duplicated_set1 = annofabcli.utils.duplicated_set(label_names1)
        duplicated_set2 = annofabcli.utils.duplicated_set(label_names2)

        flag = False
        if len(duplicated_set1) > 0:
            diff_message += f"{self.project_title1}のラベル名(en)が重複しています。{duplicated_set1}\n"
            flag = True

        if len(duplicated_set2) > 0:
            diff_message += f"{self.project_title2}のラベル名(en)が重複しています。{duplicated_set2}\n"
            flag = True

        return flag, diff_message

    def diff_labels_of_annotation_specs(
        self, labels1: List[Dict[str, Any]], labels2: List[Dict[str, Any]]
    ) -> DiffResult:
        """
        アノテーションラベル情報の差分を表示する。ラベル名(英語)を基準に差分を表示する。
        以下の項目は無視して比較する。
         * label_id
         * additional_data_definition_id
         * choice_id
        Args:
            labels1: 比較対象のラベル情報
            labels2: 比較対象のラベル情報

        Returns:
            Tuple(差分があるかどうか, 差分メッセージ)

        """
        logger.info("=== アノテーションラベル情報の差分 ===")

        diff_message = ""

        label_names1 = [AnnofabApiFacade.get_label_name_en(e) for e in labels1]
        label_names2 = [AnnofabApiFacade.get_label_name_en(e) for e in labels2]

        # 重複チェック
        is_duplicated, duplicated_message = self.validate_duplicated(label_names1, label_names2)
        diff_message += duplicated_message
        if is_duplicated:
            diff_message += f"ラベル名(en)が重複しているので、アノテーションラベル情報の差分は確認しません。\n"

        if label_names1 != label_names2:
            diff_message += (
                f"### ラベル名(en)のListに差分あり\n" f"label_names1: {label_names1}\n" f"label_names2: {label_names2}\n"
            )

            # 両方に存在するlabel_nameのみ確認する
            is_different = True
            label_names = list(set(label_names1) & set(label_names2))
        else:
            is_different = False
            label_names = label_names1

        for label_name in label_names:

            def get_label_func(label_name: str, label: Dict[str, Any]):
                return AnnofabApiFacade.get_label_name_en(label) == label_name

            label1 = more_itertools.first_true(labels1, pred=functools.partial(get_label_func, label_name))
            label2 = more_itertools.first_true(labels2, pred=functools.partial(get_label_func, label_name))
            assert label1 is not None
            assert label2 is not None

            diff_result = list(dictdiffer.diff(create_ignored_label(label1), create_ignored_label(label2)))
            if len(diff_result) > 0:
                is_different = True
                diff_message += f"ラベル名(en): {label_name} は差分あり\n" f"{pprint.pformat(diff_result)}\n"

            else:
                logger.debug(f"ラベル名(en): {label_name} は同じ")

        if not is_different:
            logger.info("アノテーションラベルは同じ")

        return is_different, diff_message

    @staticmethod
    def diff_inspection_phrases(
        inspection_phrases1: List[Dict[str, Any]], inspection_phrases2: List[Dict[str, Any]]
    ) -> DiffResult:
        """
        定型指摘の差分を表示する。定型指摘IDを基準に差分を表示する。

        Args:
            inspection_phrases1: 比較対象の定型指摘List
            inspection_phrases2: 比較対象の定型指摘List

        Returns:
            Tuple(差分があるかどうか, 差分メッセージ)

        """
        logger.info("=== 定型指摘の差分 ===")

        diff_message = ""

        # 定型指摘は順番に意味がないので、ソートしたリストを比較する
        sorted_inspection_phrases1 = sorted_inspection_phrases(inspection_phrases1)
        sorted_inspection_phrases2 = sorted_inspection_phrases(inspection_phrases2)

        phrase_ids1 = [e["id"] for e in sorted_inspection_phrases1]
        phrase_ids2 = [e["id"] for e in sorted_inspection_phrases2]

        if phrase_ids1 != phrase_ids2:
            diff_message += (
                f"### 定型指摘IDのListに差分あり\n"
                f"set(phrase_ids1) - set(phrase_ids2) = {set(phrase_ids1) - set(phrase_ids2)}\n"
                f"set(phrase_ids2) - set(phrase_ids1) = {set(phrase_ids2) - set(phrase_ids1)}\n"
            )
            return True, diff_message

        is_different = False
        for phrase1, phrase2 in zip(sorted_inspection_phrases1, sorted_inspection_phrases2):
            diff_result = list(dictdiffer.diff(phrase1, phrase2))
            if len(diff_result) > 0:
                is_different = True
                diff_message += f"定型指摘に: {phrase1['id']} は差分あり\n" f"{pprint.pformat(diff_result)}\n"

        if not is_different:
            logger.info("定型指摘は同じ")

        return is_different, diff_message

    def diff_annotation_specs(self, project_id1: str, project_id2: str, diff_targets: List[DiffTarget]) -> DiffResult:
        """
        プロジェクト間のアノテーション仕様の差分を表示する。
        Args:
            project_id1: 比較対象のプロジェクトのproject_id
            project_id2: 比較対象のプロジェクトのproject_id
            diff_targets: 比較対象の項目

        Returns:
            差分メッセージ

        """

        diff_message = ""
        is_different = False

        annotation_specs1, _ = self.service.api.get_annotation_specs(project_id1)
        annotation_specs2, _ = self.service.api.get_annotation_specs(project_id2)

        if DiffTarget.INSPECTION_PHRASES in diff_targets:
            bool_result, message = self.diff_inspection_phrases(
                annotation_specs1["inspection_phrases"], annotation_specs2["inspection_phrases"]
            )
            is_different = is_different or bool_result
            diff_message += message

        if DiffTarget.ANNOTATION_LABELS in diff_targets:
            bool_result, message = self.diff_labels_of_annotation_specs(
                annotation_specs1["labels"], annotation_specs2["labels"]
            )
            is_different = is_different or bool_result
            diff_message += message

        return is_different, diff_message

    def diff_project_settingss(self, project_id1: str, project_id2: str) -> DiffResult:
        """
        プロジェクト間のプロジェクト設定の差分を表示する。
        Args:
            project_id1: 比較対象のプロジェクトのproject_id
            project_id2: 比較対象のプロジェクトのproject_id


        Returns:
            Tuple(差分があるかどうか, 差分メッセージ)

        """
        logger.info("=== プロジェクト設定の差分 ===")

        diff_message = ""

        config1 = self.service.api.get_project(project_id1)[0]["configuration"]
        config2 = self.service.api.get_project(project_id2)[0]["configuration"]

        # ignored_key = {"updated_datetime", "created_datetime", "project_id"}
        diff_result = list(dictdiffer.diff(config1, config2))
        if len(diff_result) > 0:
            diff_message += f"### プロジェクト設定に差分あり\n" f"{pprint.pformat(diff_result)}\n"
            return True, diff_message
        else:
            logger.info("プロジェクト設定は同じ")
            return False, diff_message

    def validate_projects(self, project_id1: str, project_id2: str):
        """
        適切なRoleが付与されているかを確認する。

        Args:
            project_id1:
            project_id2:

        Returns:

        Raises:
            AuthorizationError: 自分自身のRoleがいずれかのRoleにも合致しなければ、AuthorizationErrorが発生する。
        """
        roles = [ProjectMemberRole.OWNER, ProjectMemberRole.ACCEPTER, ProjectMemberRole.TRAINING_DATA_USER]
        super().validate_project(project_id1, roles)
        super().validate_project(project_id2, roles)

    def diff(self, project_id1: str, project_id2: str, diff_targets: List[DiffTarget]) -> DiffResult:
        self.validate_projects(project_id1, project_id2)

        logger.info(f"=== {self.project_title1}({project_id1}) と {self.project_title2}({project_id2}) の差分を表示")

        diff_message = ""
        is_different = False

        if DiffTarget.MEMBERS in diff_targets:
            bool_result, message = self.diff_project_members(project_id1, project_id2)
            is_different = is_different or bool_result
            diff_message += message

        if DiffTarget.SETTINGS in diff_targets:
            bool_result, message = self.diff_project_settingss(project_id1, project_id2)
            is_different = is_different or bool_result
            diff_message += message

        if {DiffTarget.ANNOTATION_LABELS, DiffTarget.INSPECTION_PHRASES} <= set(diff_targets):
            bool_result, message = self.diff_annotation_specs(project_id1, project_id2, diff_targets)
            is_different = is_different or bool_result
            diff_message += message

        if is_different:
            diff_message = (
                f"!!! {self.project_title1}({project_id1}) と "
                f"{self.project_title2}({project_id2}) に差分あり\n" + diff_message
            )
        return is_different, diff_message

    def main(self):
        args = self.args
        project_id1 = args.project_id1
        project_id2 = args.project_id2

        diff_targets = [DiffTarget(e) for e in args.target]
        _, diff_message = self.diff(project_id1, project_id2, diff_targets)
        print(diff_message)


def parse_args(parser: argparse.ArgumentParser):
    parser.add_argument("project_id1", type=str, help="比較対象のプロジェクトのproject_id")

    parser.add_argument("project_id2", type=str, help="比較対象のプロジェクトのproject_id")

    choices = [DiffTarget.ANNOTATION_LABELS, DiffTarget.INSPECTION_PHRASES, DiffTarget.MEMBERS, DiffTarget.SETTINGS]

    parser.add_argument(
        "--target",
        type=str,
        nargs="+",
        choices=[e.value for e in choices],
        default=["annotation_labels", "inspection_phrases", "members", "settings"],
        help="比較する項目。指定しなければ全項目を比較する。"
        "annotation_labels: アノテーション仕様のラベル情報, "
        "inspection_phrases: 定型指摘,"
        "members: プロジェクトメンバ,"
        "settings: プロジェクト設定,",
    )

    parser.set_defaults(subcommand_func=main)


def main(args):
    service = build_annofabapi_resource_and_login(args)
    facade = AnnofabApiFacade(service)
    DiffProjecs(service, facade, args).main()


def add_parser(subparsers: argparse._SubParsersAction):
    subcommand_name = "diff"
    subcommand_help = "プロジェクト間の差分を表示する。"
    description = "プロジェクト間の差分を表示する。" + "ただし、AnnoFabで生成されるIDや、変化する日時などは比較しない。"
    epilog = "オーナ、チェッカーロールのいずれかを持つユーザで実行してください。"

    parser = annofabcli.common.cli.add_parser(subparsers, subcommand_name, subcommand_help, description, epilog=epilog)
    parse_args(parser)
